c-----------------------------------------------------------------------

c RLIB - a library of subprograms called by VERTEX, FRENDLY, ACTCOR
c BUILD, ADIABAT, and ISOCHOR.

c Copyright (c) 1990 by James A. D. Connolly, Institute for Mineralogy
c & Petrography, Swiss Federal Insitute of Technology, CH-8092 Zurich,
c SWITZERLAND. All rights reserved.
 
c Please do not distribute this source.

c-----------------------------------------------------------------------

      subroutine gexces (id,dg)
c-----------------------------------------------------------------------
c gexces evaluates the contribution to the gibbs energy of a pseudocompound
c arising from configurational entropy and excess properties.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst304 /sxs(k13),exces(3,k1),jend(h9,k12),ixp(k1),ifp(k1)
     *      / cst5 /p,t,xco2,u1,u2,tr,pr,r,ps/ cst11 /f(2)
     *      / cst12 /cp(k5,k1)
c-----------------------------------------------------------------------
c                                 two cases:

c                                 normal solution:
      if (ifp(id).eq.0) then 

         dg = exces(1,id) + t * exces(2,id) + p * exces(3,id) 
       
      else if (pr.eq.0d0.or.tr.eq.0d0.or.
     *         cp(1,id).eq.0d0.and.cp(2,id).eq.0d0) then
         dg = 0d0
c                                     if pr or tr = 0 then 
c                                     this is a special calculation
c                                     and we can skip this part.
      else if (ifp(id).eq.1) then
c                                     modification for fluid
c                                     projections
         xco2 = cp(2,id)

         call cfluid (fo2,fs2)

         dg = r*t*(cp(1,id)*f(1) + cp(2,id)*f(2))

      else if (ifp(id).eq.23) then

         call toop (id,dg)
 
      end if

      end 

      subroutine gvlaar1 (jd,id,dg)
c-----------------------------------------------------------------------
c gvlaar1 evaluates the contribution to the gibbs energy of a pseudocompound
c arising from van laar excess properties. gvlaar1 is less efficient than
c gvlaar, which should be used to evaluate g of all cpds simultaneously
c at one p-t-mu-x condition.
c
c input:
c      jd - solution pointer
c      id - compound pointer
c output:
c      dg - excess energy     
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      double precision phi(msp**3), tphi

      common/ cst304 /sxs(k13),exces(3,k1),jend(h9,k12),ixp(k1),ifp(k1)
     * / cst139 /wgl(h9,27,3),vlar(h9,msp**3,3),jsub(h9,27,2),jterm(h9)
     * / cst5 /p,t,xco2,u1,u2,tr,pr,r,ps

c-----------------------------------------------------------------------
c                            first compute "volumes"
      tphi = 0d0

      do i = 1, jend(jd,1)
         phi(i) =  (vlar(jd,i,1) + t * vlar(jd,i,2) 
     *                           + p * vlar(jd,i,3))* sxs(ixp(id)+i)
         tphi = tphi + phi(i) 
      end do 
c                            now get the excess energy, initialize
c                            with configurational entropy
      dg = t*exces(2,id)
c
      do i = 1, jterm(jd)
c                            assume holland powell form, all terms regular
         dg = dg + (wgl(jd,i,1) + t * wgl(jd,i,2) + p * wgl(jd,i,3)) 
     *           * phi(jsub(jd,i,1)) * phi(jsub(jd,i,2))/tphi**2
      end do  

      end 

      function gzero (id)
c-----------------------------------------------------------------------
c gzero computes the reference pressure free energy of a compound 
c with no transitions identified by the argument 'id'. see gcpd. 
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst1 /thermo(k4,k10),uf(2),us(h5)/ cst12 /cp(k5,k1)
     *      / cst5 /p,t,xco2,um(2),tr,pr,r,ps/ cst44 /dlogt,dsqrtt
     *      / cst307 /jsat,ksat,jfct,kfct,jmct,jprct
 
      gzero = thermo(1,id)
c                                    -sdt
     *      + t * (thermo(2,id) - thermo(4,id) * dlogt
     *      - t * (thermo(5,id) + thermo(7,id) * t))
     *      - (thermo(6,id) + thermo(10,id) / t) / t
     *      + thermo(8,id) * dsqrtt + thermo(9,id)*dlogt
 
      do j = 1, jmct      
c                                    -ndu   
         gzero = gzero - cp(jprct+j,id) * um(j)
      end do

      end 

      subroutine gphase (id,gph)
c-----------------------------------------------------------------------
c gphase computes the gibbs free energy of a compound identified by index
c id, it is assumed that the variables in cst44 are set prior to the call.
c gphase does not assume that the properties of a pseudocompound endmembers
c are known (unlike gall) it is thus less efficient than gall.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst304 /sxs(k13),exces(3,k1),jend(h9,k12),ixp(k1),ifp(k1)
      common/ cst60 /ipoint,imyn
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar

      if (id.le.ipoint) then
c                              the phase is an endmember compound
         call gcpd (id,gph)

      else
c                              the phase is a pseudocompound, first
c                              compute endmember properties:
         gph = 0d0

         if (ifp(id).ne.1) then
c                              compute mech mix G for 
c                              all models except fluid 
            do k = 1, jend(ikp(id),1) 
               call gcpd (jend(ikp(id),2+k),dg)
               gph = gph + dg*sxs(ixp(id)+k)
            end do 
         else 
c                              excess props don't include vdp:
            do k = 1, jend(ikp(id),1) 
               gph = gph + gzero(jend(ikp(id),2+k))*sxs(ixp(id)+k)
            end do 
         end if 
c                              now get the excess and/or ideal mixing effect:
         
         if (ifp(id).ne.2) then 
             call gexces (id,dg)
         else 
c                              for van laar
            call gvlaar1 (ikp(id),id,dg)
         end if 
         
         gph = gph + dg

      end if 

      end

      subroutine gcpd (id,gval)
c-----------------------------------------------------------------------
c gcpd computes the gibbs free energy of a compound identified by
c the arguement 'id' from the thermochemical parameters stored
c in the array 'thermo' which is located in common block 'cst1'.
c the parameters are: g(pr,tr) [thermo(1,id)], s(pr,tr) [thermo
c (2,id)], v(pr,tr) [thermo(3,id)], the coefficients of the extended
c maier-kelley heat capacity equation:
 
c           cp(pr,t)=a+b*t+c/(t*t)+d*t*t+e/t**(1/2)+f/t+g/t**3
 
c [thermo(4-10,id)], and the coefficients of the volumetric 
c equations given in the program documentation (Eqs 2.1-2.3):

c        b8 = 0 =>
 
c           v(p,t) = v(pr,tr) + b2*(t-tr) + b4*(p-pr)
c                           + b6*(p-pr)**2 + b7*(t-tr)**2

c        b8 < 0 =>

c           v(p,t) = v(pr,tr) * exp[ b3*(t-tr) + b8*(p-pr) ]

c        b8 > 0 =>

c           v(p,t) = v(pr,t) * [1 + b8*p/(b8*p+Kt)]^(1/b8)
c               Kt = b6 + b7 * t
c            alpha = b1 + 2*b2*t + b3/t - b4/t^2 + b5/t^(1/2)      

c [thermo(11-18,id)].

c the variables in cst44 must be set before a call to gcpd.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst1 /thermo(k4,k10),uf(2),us(h5)
     *      / cst11 /f(2)
     *      / cst10 /iff(2),idss(h5),ifug,ifyn,isyn/ cst113 /ifproj
     *      / cst5 /p,t,xco2,um(2),tr,pr,r,ps/ cst12 /cp(k5,k1)
     *      / cst203 /therdi(m8,m9),therlm(m7,m6,k9)
     *      / cst204 /ltyp(k1),lmda(k1),idis(k1)/ cst44 /dlogt,dsqrtt
     *      / cst307 /jsat,ksat,jfct,kfct,jmct,jprct

      save rkt

      data rkt/0d0/
 
      gval = thermo(1,id)
c                                    -sdt
     *       + t * (thermo(2,id) - thermo(4,id) * dlogt
     *       - t * (thermo(5,id) + thermo(7,id) * t))
     *       - (thermo(6,id) + thermo(10,id) / t) / t
     *       + thermo(8,id) * dsqrtt + thermo(9,id)*dlogt
c                                     vdp-ndu term:
      if (thermo(18,id).eq.0d0) then 
c                                     normal polynomial:
         vdp =  p * (thermo(3,id) 
     *               + (thermo(17,id) * t + thermo(12,id)) * t
     *               + (thermo(16,id) * p + thermo(14,id)) * p)

      else if (thermo(18,id).gt.0d0) then
c                                     murghnahan EoS:
c                                     vdp = V(1,T)*KT**(1/K')/(K'-1)
c                                           * [(KT+K'*p)**(1-1/K') -
c                                              (KT+K'*pr)**(1-1/K')]
         rkt = thermo(16,id) + thermo(17,id)*t
c                                     a fuckwit has entered a ridiculous
c                                     temperature
         if (rkt.lt.0d0) call error (46,t,id,'GCPD') 

         vdp = (thermo(3,id) + (thermo(11,id) + thermo(12,id)*t)*t +
     *          thermo(13,id)*dlogt + thermo(14,id)/t + 
     *          thermo(15,id)*dsqrtt) * rkt**thermo(21,id)/thermo(22,id) 
     *          *((rkt+thermo(18,id)*p)**thermo(19,id)
     *           -(rkt+thermo(20,id))**thermo(19,id))

      else 
c                                     gottschalk.
         vdp = thermo(11,id)*dexp(thermo(13,id)*t)*
     *               (1d0 - dexp(thermo(18,id)*(p - pr)))

      end if
c                                    -ndu term
      du = 0d0
      do j = 1, jmct      
         du = du - cp(jprct+j,id) * um(j)
      end do

      gval = gval + vdp + du
c                                     check for lambda transitions:
      if (ltyp(id).ne.0) then
 
         if (ltyp(id).lt.4) then
c                                     ubc-type transitions
            call lamubc (p,t,dg,lmda(id),ltyp(id))
            gval = gval + dg
 
         else if (ltyp(id).lt.7) then
c                                     standard transitions
            call lamhel (p,t,gval,vdp+du,lmda(id),ltyp(id))
 
         else if (ltyp(id).lt.10) then
c                                     supcrt q/coe lambda transition
            call lamqtz (p,t,gval,du,lmda(id),id)
 
         else if (ltyp(id).gt.9 .and. ltyp(id).le.12 ) then
c                                     holland and powell (landau model)
            call lamlan (dg,rkt,lmda(id),ltyp(id))
            gval = gval + dg
 
         end if
      end if
c                                     check for temperature dependent
c                                     order/disorder:
      if (idis(id).ne.0) then
         jd = idis(id)
         call disord (t,therdi(1,jd),therdi(2,jd),therdi(3,jd),
     *                  therdi(4,jd),therdi(5,jd),therdi(6,jd),
     *                  therdi(7,jd),therdi(8,jd),therdi(9,jd),gss)
         gval = gval + gss
      end if
c                                     modification for fluid
c                                     projections:
      if (ifproj.gt.0.and.thermo(3,id).eq.0d0) then
c                                     if pr or tr = 0 then 
c                                     this is a special calculation
c                                     and we can skip this part.
         if (pr.eq.0d0.or.tr.eq.0d0) goto 999

         if (cp(1,id).eq.0d0.and.cp(2,id).eq.0d0) goto 999

         if (ifproj.eq.1) then
            xco2 = 0d0
            call cfluid (fo2,fs2)
            gval = gval + r*t*cp(1,id)*f(1)
         else if (ifproj.eq.2) then
            xco2 = 1d0
            call cfluid (fo2,fs2)
            gval = gval + r*t*cp(1,id)*f(2)
         else 
            xco2 = cp(2,id)
            call cfluid (fo2,fs2)
            gval = gval + r*t*(cp(1,id)*f(1) + cp(2,id)*f(2))
         end if  
      end if
 
999   end

      subroutine rmodel (tname,istot)
c---------------------------------------------------------------------
c rmodel - reads solution models from LUN n9.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
  
      include 'perplex_parameters.h'
 
      character*10 mname*8,tname,fname,names*8
 
      common/ cst108 /wg(27,11),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(msp,msp,msp),isub(27,9,2),imd(3),
     *      imsol(msp,msp,msp),ijk(mst),insp(mst,msp),jsp(mst),
     *      jsitp(mst),iosp(mst),jst(mst),ist(mst),
     *      isp(mst),isite,ispct(mst,msp),jnsp(mst,msp),jsite,iterm,iord
     *      / cst107 /a0(5,5),acoef(5,5,m0),smult(5),ecoef(msp,msp,msp),
     *      nsite,nspm1(5),nterm(5,5),
     *      nsub1(5,5,m0,4),nsub2(5,5,m0,4),nttyp(5,5,m0)
     *      / cst8 /names(k1)/ cst221 /vlaar(3,msp,msp,msp),jsmod
     *      / csta7 /fname(h9)/ cst18a /mname(msp,msp,msp)/ cst79 /isoct
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst140 /inew

      istot = 0
c                               model indicator 0 - fluid
c                                               1 - margules or ideal
c                                               2 - van laar
c                                               3 - speciation, margules
c                                               4 - speciation, van laar
10    read (n9, 1000, end = 99) tname
c                               if blank, assume comment.
      if (tname.eq.' ') goto 10

      if (inew.eq.0) then 
         read (n9,*,err=90) jsmod
      else
         jsmod = 1
      end if 
c                               read number of independent sites:
      read (n9,*,err=90) isite

      do i = 1, mst
         isp(i) = 1
         ist(i) = 1
      end do 
c                               read number of species on each site:
      read (n9,*,err=90) (isp(i),ist(i),i=1,isite)
c                               total number of endmembers:
      istot = 1
      do i = 1, isite
         istot = istot*isp(i)
      end do 
c                               read endmember names:
      read (n9,1010) (((mname(i,j,k), i= 1,isp(1)),
     *                                j= 1,isp(2)), k= 1,isp(3))
c                               read endmember flags:
      read (n9,*,err=90) ((( iend(i,j,k), i= 1,isp(1)),
     *                                j= 1,isp(2)), k= 1,isp(3))
c                               read composition limits:
      do i = 1, isite

         read (n9,*,err=90) (xmn(i,j),xmx(i,j),xnc(i,j), 
     *                           j= 1, isp(i)-1),imd(i)
c                               valid subdivision choice?:
         if (imd(i).gt.10) call error (323,a0(1,1),i,'PRIME9')
      end do 
c                               read total number of terms, and order
c                               of highest order term, in mixing model
c                               ideal models: iterm = iord = 0.
      read (n9,*,err=90) iterm, iord
      if (iterm.eq.0) goto 40
c                               read parameters for margules
c                               expression of the excess free
c                               energy of the solution.
      do h = 1, iterm
         read (n9,*,err=90) ((isub(h,i,j), j = 1, 2), i = 1, iord)
         read (n9,*,err=90) (wg(h,k), k = 1, 11)
      end do 
c                              expansion for S(configurational)
40    read (n9,*,err=90) nsite

      if (nsite.ne.0) call smodel (tname)
c                              read p-t dependent size parameter for 
c                              each endmember
      read (n9,*,err=90) idummy
      if (jsmod.eq.2) read (n9,*,err=90) (((
     *                       (vlaar(l,i,j,k), l= 1, 3), i= 1,isp(1)),
     *                                    j= 1,isp(2)), k= 1,isp(3))

      goto 99

90    call error (20,wg(1,1),i,tname)

1000  format (a10)
1010  format (3(a8,1x))

99    end 

      subroutine zeroi (iarray,index,ivalue,n)

      integer iarray(n), n, index, ivalue

      do i = 1, index
         iarray(i) = ivalue
      end do 

      end

      subroutine cmodel (im,idsol,tname,istot,jstot,ibuild,x1,x2)
c---------------------------------------------------------------------
c cmodel - checks to see if solution models contain valid endmembers
c and, if not, whether they can be recast as simpler models.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
  
      include 'perplex_parameters.h'
 
      character*10 mname*8,tname,fname,names*8,missin(msp*msp*msp)*8,
     *             x1*8,x2*8

      integer imiss
 
      common/ cst108 /wg(27,11),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(msp,msp,msp),isub(27,9,2),imd(3),
     *      imsol(msp,msp,msp),ijk(mst),insp(mst,msp),jsp(mst),
     *      jsitp(mst),iosp(mst),jst(mst),ist(mst),
     *      isp(mst),isite,ispct(mst,msp),jnsp(mst,msp),jsite,iterm,iord
     *      / cst18a /mname(msp,msp,msp)
     *      / cst8 /names(k1)/ cst60 /ipoint,imyn/ cst79 /isoct
     *      / cst204 /ltyp(k1),lmda(k1),idis(k1)/ csta7 /fname(h9)
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar    

      jstot = 0
c                              if called by build (ibuild = 1) skip the
c                              name check:
      if (ibuild.eq.1) goto 60
c                              check that solution name matches a 
c                              name requested in input from n1
      do i = 1, isoct
         if (tname.eq.fname(i)) then 
c                              got a match, goto 60:
            idsol = i 
            im = im + 1
            goto 60 
         end if 
c
      end do 
c                              didn't find a match, read a new name:
      goto 99 
c                              check that the endmembers match with data
c                              from n2:   
c                              disable fixed activity corrections:
60    do i = 1, mst
         jst(i) = 1
         do j = 1, msp
            ispct(i,j) = 0
            insp(i,j) = 1
         end do 
      end do 
 
      do i = 1, msp 
         do j = 1, msp
            do k = 1, msp
               imsol(i,j,k) = 0
            end do 
         end do 
      end do 

      do 70 h = 1, ipoint
         do i = 1, isp(1)
            do j = 1, isp(2)
               do 100 k = 1, isp(3)

                  ijk(1) = i
                  ijk(2) = j
                  ijk(3) = k 
c                               in build, check for forbidden choices:
                  if (ibuild.eq.1) then
                     if (mname(i,j,k).eq.x1.or.
     *                   mname(i,j,k).eq.x2) then
                        write (*,1010) tname, x1, x2
                        jstot = 0
                        goto 99
                      end if
                   end if

                  if (names(h).ne.mname(i,j,k)) goto 100

                  jstot = jstot + 1 
                  imsol(i,j,k) = h
c 
                  do i1 = 1, isite
                     do j1 = 1, isp(i1)
                        if (ijk(i1).eq.j1) ispct(i1,j1)=ispct(i1,j1)+1
                     end do 
                  end do 
c                                set kill flag: 
                  if (iend(i,j,k).ne.0) ikp(h) = -1

c                                found all endmembers:
                  if (jstot.eq.istot) goto 99

                  goto 70
  100          continue
            end do 
         end do
70    continue
c                                missing endmember warnings:
      if (jstot.lt.2) then
         im = im - 1
         call warn (25,wg(1,1),jstot,tname)
         jstot = 0 
      else 
         imiss = 0 
         do i = 1, isp(1)
            do j = 1, isp(2)
               do k = 1, isp(3)
                  if (imsol(i,j,k).eq.0) then
                     imiss = imiss + 1
                     missin(imiss) = mname(i,j,k)
                  end if 
                end do 
             end do
          end do 

          write (*,1000) tname,(missin(i),i=1,imiss)
      end if 

1000  format ('**warning ver114** the following endmembers',
     *          ' are missing for ',a,/,4(8(2x,a),/))
1010  format ('**warning ver113** ',a,' is not a valid model',
     *        ' because component ',a,' or ',a,/,
     *        ' is constrained or missing.')
99    end

      subroutine emodel (tname,istot)
c---------------------------------------------------------------------
c emodel - attempts to recast solution models as simpler models.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
  
      include 'perplex_parameters.h'
 
      character*8 tname*10, names

      integer lmn(3)
  
      common/ cst108 /wg(27,11),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(msp,msp,msp),isub(27,9,2),imd(3),
     *      imsol(msp,msp,msp),ijk(mst),insp(mst,msp),jsp(mst),
     *      jsitp(mst),iosp(mst),jst(mst),ist(mst),
     *      isp(mst),isite,ispct(mst,msp),jnsp(mst,msp),jsite,iterm,iord
     *      / cst8 /names(k1)
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar

      do i = 1, mst
         iosp(i) = 0
         do j = 1, msp
            insp(i,j) = 0
         end do
      end do 
c                                 look for sites where there
c                                 are enough endmembers, assuming
c                                 that the count of a good species
c                                 should be
      if (isite.gt.1) then

      do 410 i = 1, isp(1)
         ijk(1) = i
         do 420 j = 1, isp(2)
            ijk(2) = j 
            do 430 k = 1, isp(3)
            ijk(3) = k

            if (imsol(i,j,k).ne.0) then
c                                 check if the endmember can 
c                                 make a binary:
               do 440 i1 = 1, isp(1)
                  lmn(1) = i1
                  do 450 j1 = 1, isp(2)
                     lmn(2) = j1
                     do 460 kk = 1, isp(3)
                        lmn(3) = kk
                        if (imsol(i1,j1,kk).ne.0) then
                           do 470 i2 = 1, isite
                              if (ijk(i2).ne.lmn(i2)) then
                                 do 480 i3 = 1, isite
                                    if (i3.eq.i2) goto 480
                                    if (ijk(i3).ne.lmn(i3)) goto 460
c                                  species ijk(i2) and lmn(i2) are
c                                  valid on site i2:
                                     insp(i2,ijk(i2)) = 1
                                     insp(i2,lmn(i2)) = 1
480                               continue
                               end if 
470                         continue
                         end if
460                  continue
450               continue
440            continue
            end if
430         continue
420      continue
410   continue
 
      else 

         do j = 1, isp(1)
            if (ispct(1,j).gt.0) insp(1,j) = 1
         end do 
 
      end if
c                                 count the number of endmembers
c                                 left:
600   do i = 1, mst
         jsp(i) = 0
         do j = 1, msp
            jnsp(i,j) = 1
         end do 
      end do 
c                                 and make a set of valid species
      do i = 1, isite
         do j = 1, isp(i)
            if (insp(i,j).ne.0) then 
               jsp(i) = jsp(i) + 1
               jnsp(i,j) = jsp(i)
            end if 
         end do 
      end do 

      jstot = 0 

      do i = 1, isp(1)
         do j = 1, isp(2)
            do 20 k = 1, isp(3)

               if (imsol(i,j,k).eq.0) goto 20
               ijk(1) = i
               ijk(2) = j
               ijk(3) = k 
               do l = 1, isite
                  if (jsp(l).ne.0.and.insp(l,ijk(l)).eq.0) then
c                                 endmember is not valid:
                     if (iend(i,j,k).eq.1) ikp(imsol(i,j,k)) = 0
                     imsol(i,j,k) = 0
                   end if 
                end do 
          
                jstot = jstot + 1

20          continue
         end do 
      end do 

      kstot = 1
      do i = 1, mst
         if (jsp(i).eq.0) jsp(i) = 1
         kstot = kstot * jsp(i)
         jsitp(i) = i
      end do 

      if (kstot.lt.2) then
         call warn (34,wg(1,1),i,tname)
         istot = 0
      end if 
c                                 if jsp(1)*jsp(2)*jsp(3) > jstot
c                                 it's possible to make several
c                                 different solutions, eliminate
c                                 a species on first site possible
c                                 and try again:
      if (kstot.gt.jstot) then

         call warn (33,wg(1,1),i,tname)

         imin = 100

         do i = 1, isite
            do j = 1, isp(i)
               if (jsp(i).gt.1.and.insp(i,j).ne.0.and.
     *             ispct(i,j).lt.imin) then

                  imin = ispct(i,j)
                  ik = i
                  jk = j

               end if
            end do
         end do 

         insp(ik,jk) = 0

         goto 600

      end if         
c                                 make a pointer for old site
c                                 index to new:
      jsite = 0
      jmax = isite

      do i = 1, isite
         if (jsp(i).eq.1) then
            jsitp(i) = jmax
            jst(jmax) = 1
            jmax = jmax - 1
         else 
            jsite = jsite + 1
            jsitp(i) = jsite
            jst(jsite) = ist(i)
         end if
      end do 
c                                locate non-zero species id's
      do i = 1, isp(1)
         do j = 1, isp(2)
            do 100 k = 1, isp(3)
            if (imsol(i,j,k).eq.0) goto 100
            ijk(1) = i
            ijk(2) = j 
            ijk(3) = k 
               do l = 1, mst
                  if (isp(l).gt.1.and.jsp(l).eq.1
     *              .and.insp(l,ijk(l)).eq.0) iosp(l) = ijk(l)
               end do 
100         continue
          end do 
      end do 
c                                if this works, then the next loop
c                                could be lot simpler

c                                relocate endmembers:
      do 40 i = 1, isp(1)
 
         if (i.ne.iosp(1).and.insp(1,i).eq.0) goto 40
         ijk(jsitp(1)) = jnsp(1,i)

         do 50 j = 1, isp(2)

            if (j.ne.iosp(2).and.jsite.gt.1.and.insp(2,j).eq.0) 
     *                                                      goto 50
            ijk(jsitp(2)) = jnsp(2,j)

            do 60 k = 1, isp(3)

               if (k.ne.iosp(3).and.jsite.gt.2.and.insp(3,k).eq.0)
     *                                                      goto 60
               ijk(jsitp(3)) = jnsp(3,k)
               if (imsol(i,j,k).eq.0) goto 60
               imsol(ijk(1),ijk(2),ijk(3)) = imsol(i,j,k)
60          continue
50       continue
40    continue
c                                relocate compositional limits:
      do i = 1, isite
         if (jsp(i).gt.1) then
            do j = 1, jsp(i) - 1
               xmx(jsitp(i),j) = xmx(i,j)
               xmn(jsitp(i),j) = xmn(i,j)
               xnc(jsitp(i),j) = xnc(i,j)
            end do 

            if (jsp(i).eq.2.and.isp(i).eq.3.and.imd(i).gt.0) then

               imd(jsitp(i)) = 0

               if (xnc(jsitp(i),j).eq.0.or.xnc(jsitp(i),j).gt.
     *             xmx(jsitp(i),j)-xmn(jsitp(i),j)) xnc(jsitp(i),j) =
     *             (xmx(jsitp(i),j)-xmn(jsitp(i),j))/10.
  
               write (*,1140) tname,jsitp(i),i,
     *                        imd(jsitp(i)),xmx(jsitp(i),1),
     *                        xmn(jsitp(i),1),xnc(jsitp(i),1)

            else 

               imd(jsitp(i)) = imd(i)

            end if 

         end if 
      end do 
c                                redefine the other counters:  
      istot = kstot 
      do i = 1, mst
         isp(i) = 1
      end do

      do i = 1, isite
         ist(i) = jst(jsitp(i))
         isp(i) = jsp(jsitp(i))
      end do 

      isite = jsite
c                               check for unknown screw up
      do i = 1, isp(1)
         do j = 1, isp(2)
            do k = 1, isp(3)
               if (imsol(i,j,k).eq.0) then 
                  istot = 0 
                  call warn (34,wg(1,1),i,tname)
                  goto 99 
               end if
            end do 
         end do 
      end do    
c                                write warning:
      write (*,2000) tname
      write (*,2010) (((i,j,k,names(imsol(i,j,k)), 
     *                  i = 1, isp(1)),j = 1, isp(2)), k = 1, isp(3))

1140  format ('**warning ver117** the subdivision scheme for '
     *        ,a10,' will be interpreted',/,'as a binary model on site'
     *        ,1x,i1,' (originally site ',i1,') with:',/,t12,'imod=',
     *        1x,i1,/,t12,'xmax= ',f7.3,/,t12,'xmin= ',f7.3,/,t12,
     *        'xinc= ',f7.3,/,'To change this, or if an',
     *        ' error (probably from routine CART) follows,',/,
     *        'modify the solution model (documentation sect. 4.1)')
2000  format ('**warning ver501** ',a, 
     *          ' will be recast with endmembers:')
2010  format (10x,5(i1,i1,i1,' - ',a8,'   '))

99    end 

      subroutine nmodel 
c----------------------------------------------------------------------
c nmodel -  nmodel reassigns solution model parameters to be 
c           consistent with the endmembers chosen by subroutine emodel.
c----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
  
      include 'perplex_parameters.h'
 
      character*8 names
   
      common/ cst108 /wg(27,11),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(msp,msp,msp),isub(27,9,2),imd(3),
     *      imsol(msp,msp,msp),ijk(mst),insp(mst,msp),jsp(mst),
     *      jsitp(mst),iosp(mst),jst(mst),ist(mst),
     *      isp(mst),isite,ispct(mst,msp),jnsp(mst,msp),jsite,iterm,iord
     *      / cst8 /names(k1)/ cst221 /vlaar(3,msp,msp,msp),jsmod
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst107 /a0(5,5),acoef(5,5,m0),smult(5),ecoef(msp,msp,msp),
     *                nsite,nspm1(5),nterm(5,5),
     *                nsub1(5,5,m0,4),nsub2(5,5,m0,4),nttyp(5,5,m0)


c                                next eliminate unused terms
c                                in margules and configurational
c                                entropy expressions.
      if (iterm.ne.0) then
         jterm = 0
         do 10 h = 1, iterm
            do 20 k = 1, iord
               if (isub(h,k,1).eq.0) then 
                  goto 20
                else if (insp(isub(h,k,1),isub(h,k,2)).eq.0) then 
                  goto 10
                end if 
20          continue
c                                term contains no invalid species
c                                count it:
            jterm = jterm + 1

            do i = 1, iord
               if (isub(h,i,1).eq.0) then 
                  isub(jterm,i,1) = 0 
                  isub(jterm,i,2) = 0
               else  
                  isub(jterm,i,1) = jsitp(isub(h,i,1))
                  isub(jterm,i,2) = jnsp(isub(h,i,1),isub(h,i,2))
               end if 
             end do 

             do j = 1, 11  
                wg(jterm,j) = wg(h,j)
             end do 
10       continue
         iterm = jterm       
      end if     
c                                 now entropy terms:
      if (nsite.ne.0) then 
         lsite = 0    
         do 50 i = 1, nsite
            lsp = 0
            do 60 j = 1, nspm1(i)
               ll2 = 0  
               l4 = 0 
               if (a0(i,j).ne.0) then
c                                 the z function contains a
c                                 constant
                  if (lsp.eq.0) lsite = lsite + 1
                  lsp = lsp + 1
                  ll2 = 1
                  l4 = 1
                  a0(lsite,lsp) = a0(i,j)
                  smult(lsite) = smult(i)
               end if 
c                                 look if it contains any
c                                 valid terms
               lterm = 0
               l3 = 0

               do 70 k = 1, nterm(i,j)
      
                  lin = 0

                  do l = 1, nttyp(i,j,k)
                     n1 = nsub1(i,j,k,l)
                     n2 = nsub2(i,j,k,l)

                     if (insp(n1,n2).eq.0.and.n2.ne.iosp(n1)) goto 70
                     if (n2.eq.iosp(n1)) lin = lin + 1

                  end do 
c                                 the term contributes to the 
c                                 site fraction
                  if (lsp.eq.0) then
                     lsite = lsite + 1
                     lsp = 1
                     ll2 = 1
                     l3 = 1
                     a0(lsite,lsp) = 0d0
                     smult(lsite) = smult(i)
                  else if (l3.eq.0) then 
                     if (l4.eq.0) then
                        lsp = lsp + 1
                        a0(lsite,lsp) = 0d0
                        smult(lsite) = smult(i)
                     end if 
                     l3 = 1
                  end if                           

                  if (lin.eq.nttyp(i,j,k)) then
c                                the term contains only
c                                constants:
                     a0(lsite,lsp) = a0(lsite,lsp) + acoef(i,j,k)

                  else 
c                                the term contains at least one X.
                     lterm = lterm + 1
                     acoef(lsite,lsp,lterm) = acoef(i,j,k)
                     lin = 0 

                     do l = 1, nttyp(i,j,k)
                        n1 = nsub1(i,j,k,l)
                        n2 = nsub2(i,j,k,l)
                        if (n2.ne.iosp(n1)) then
                           lin = lin + 1
                           nsub1(lsite,lsp,lterm,lin) = jsitp(n1)
                           nsub2(lsite,lsp,lterm,lin) = jnsp(n1,n2)
                        end if
                     end do 
                     nttyp(lsite,lsp,lterm) = lin
                  end if 
70             continue
               if (lterm.gt.0.or.ll2.gt.0) nterm(lsite,lsp) = lterm
60          continue
            if (lsp.gt.0) nspm1(lsite) = lsp 
50       continue

         nsite = lsite

      end if 

      end

      subroutine smodel (tname)
c-----------------------------------------------------------------
c smodel reads models for configurational entropy
c of solutions with dependent site mixing or disordered
c endmembers, see documentation section 1.3.1, equation (8b)
c-----------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*10 tname

      common/ cst107 /a0(5,5),acoef(5,5,m0),smult(5),ecoef(msp,msp,msp),
     *                nsite,nspm1(5),nterm(5,5),
     *                nsub1(5,5,m0,4),nsub2(5,5,m0,4),nttyp(5,5,m0)
     *      / cst108 /wg(27,11),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(msp,msp,msp),isub(27,9,2),imd(3),
     *      imsol(msp,msp,msp),ijk(mst),insp(mst,msp),jsp(mst),
     *      jsitp(mst),iosp(mst),jst(mst),ist(mst),
     *      isp(mst),isite,ispct(mst,msp),jnsp(mst,msp),jsite,iterm,iord
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
c-----------------------------------------------------------------

      if (nsite.gt.5) call error (31,a0(1,1),isite,tname)
c
      if (nsite.lt.isite) call error (30,a0(1,1),isite,tname)
c                                     for each site
      do i = 1, nsite 
c                                     read # of species, and site 
c                                     multiplicty.
         read (n9,*,err=90) knsp,smult(i)
         nspm1(i) = knsp - 1
c                                     for each species, read
c                                     function to define the
c                                     site fraction of the species:
         do j = 1, nspm1(i)
c                                     read # of terms in the 
c                                     site fraction function and a0.
            read (n9,*,err=90) nterm(i,j), a0(i,j)
            if (nterm(i,j).gt.m0) call error (33,a0(1,1),m0,tname)
c                                     for each term:
            do k = 1, nterm(i,j)
c                                     read term type:
               read (n9,*,err=90) nttyp(i,j,k)
               if (nttyp(i,j,k).lt.5) then
                  read (n9,*,err=90) acoef(i,j,k),(nsub1(i,j,k,l),
     *                               nsub2(i,j,k,l),l = 1, nttyp(i,j,k))
               else
                  call error (29,a0(1,1),nttyp(i,j,k),tname)
               end if
            end do 
         end do 
      end do 

      return

90    call error (20,a0(1,1),i,tname)

      end 

      subroutine snorm (tname)
c----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*10 tname

      double precision x(mst,msp)

      common/ cst107 /a0(5,5),acoef(5,5,m0),smult(5),ecoef(msp,msp,msp),
     *                nsite,nspm1(5),nterm(5,5),
     *                nsub1(5,5,m0,4),nsub2(5,5,m0,4),nttyp(5,5,m0)
     *      / cst5 /p,t,xc,u1,u2,tr,pr,r,ps
     *      / cst108 /wg(27,11),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(msp,msp,msp),isub(27,9,2),imd(3),
     *      imsol(msp,msp,msp),ijk(mst),insp(mst,msp),jsp(mst),
     *      jsitp(mst),iosp(mst),jst(mst),ist(mst),
     *      isp(mst),isite,ispct(mst,msp),jnsp(mst,msp),jsite,iterm,iord

c                                     get normalization constants:
      do i = 1, isp(1)
c                                     set x's to endmember values:
         x(1,i) = 1d0
         do i1 = 1, isp(1)
            if (i1.ne.i) x(1,i1) = 0d0
         end do 
         do j = 1, isp(2)
            x(2,j) = 1d0
            do j1 = 1, isp(2)
               if (j1.ne.j) x(2,j1) = 0d0
            end do 
            do k = 1, isp(3)  
               x(3,k) = 1d0
               do ik1 = 1, isp(3)
                  if (ik1.ne.k) x(3,ik1) = 0d0
               end do 
c                                    x's now set, evaluate ln W:
               call omega (x,dlnw,tname)
               ecoef(i,j,k) = dlnw
            end do 
         end do 
      end do 

      write (*,2000) tname
      write (*,2010) (((i,j,k,r*ecoef(i,j,k), 
     *                  i = 1, isp(1)),j = 1, isp(2)), k = 1, isp(3))
2000  format ('Endmember configurational entropies (',
     *          'doc. eq. 8.2) for ',a,' are:')
2010  format (5(2x,i1,i1,i1,' - ',f8.5,2x))
      end 

      subroutine omega (x,dlnw,tname)
c----------------------------------------------------------------------
c subroutine to evaluate the log of the number of site configurations
c per mole of a solution (dlnw) with composition x.
c----------------------------------------------------------------------

      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*10 tname

      double precision x(mst,msp)

      common/ cst107 /a0(5,5),acoef(5,5,m0),smult(5),ecoef(msp,msp,msp),
     *                nsite,nspm1(5),nterm(5,5),
     *                nsub1(5,5,m0,4),nsub2(5,5,m0,4),nttyp(5,5,m0)

      dlnw = 0d0
c                                     for each site
      do i = 1, nsite
         zt = 0d0
c                                     get site fractions
         do j = 1, nspm1(i)
            z = a0(i,j)
            if (nterm(i,j).ne.0) then 
c                                     for each term:
               do k = 1, nterm(i,j)
                  zad = acoef(i,j,k)
                  do l = 1, nttyp(i,j,k)
                     zad = zad * x(nsub1(i,j,k,l),nsub2(i,j,k,l))
                  end do 
                  z = z + zad
               end do 
            end if 

            call zchk (z,tname)
            zt = zt + z
            if (z.gt.0d0.and.z.lt.1d0) then 
               dlnw = dlnw - smult(i) * z * dlog (z)
            end if 
            
         end do 
         z = 1d0 - zt
         call zchk (z,tname)
         if (z.gt.0d0.and.z.lt.1d0) dlnw = dlnw - smult(i) * z * dlog(z)
      end do 

      end
 
      subroutine subdiv (tname)
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      character*10 tname
 
      include 'perplex_parameters.h'

      parameter (ms1=msp-1)
 
      common/ cst86 /xy(ms1,k1),y(k1,mst,ms1),ntot,npairs
     *      / cst108 /wg(27,11),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(msp,msp,msp),isub(27,9,2),imd(3),
     *      imsol(msp,msp,msp),ijk(mst),insp(mst,msp),jsp(mst),
     *      jsitp(mst),iosp(mst),jst(mst),ist(mst),
     *      isp(mst),isite,ispct(mst,msp),jnsp(mst,msp),jsite,iterm,iord
c---------------------------------------------------------------------
c                                 do the first site:
      call cart (1,tname)
c
      ntot = npairs
      np1 = npairs

      do h = 1, np1
         do i = 1, isp(1) - 1
            y(h,1,i) = xy(i,h)
         end do 
      end do 

      if (isite.eq.1) goto 99
c                                 do the second site:
      call cart (2,tname)

c                                 there will be a total of
c                                 (npairs-1)*np1 compositions,
c                                 copy the first site 2 distribution
c                                 into the first np1 compositions:
      do h = 1, np1 
         do i = 1, isp(2) - 1
            y(h,2,i) = xy(i,1)
         end do 
      end do 

      do h = 2, npairs
c                                 for each site 2 composition,
c                                 duplicate the range of site 1
c                                 compositions.
         do i = 1, np1
            ntot = ntot + 1
            do j = 1, isp(1) - 1
               y(ntot,1,j) = y(i,1,j)
            end do 
c                                 put in the new site 2 compositions:
            do j = 1, isp(2) - 1 
               y(ntot,2,j) = xy(j,h)
            end do 
         end do 
      end do 
c                                 do the third site:
      if (isite.eq.2) goto 99

      np1 = (npairs-1) * np1

      call cart (3,tname)
c                                 copy the first site 3 distribution
c                                 into the first np1*np2 compositions:
      do h = 1, ntot
         do i = 1, isp(3) - 1
            y(h,3,i) = xy(i,1)
         end do 
      end do 
c                                 for each site 3 composition,
c                                 duplicate the range of site 1 and 
c                                 site 2 compositions.
      do h = 2, npairs

         do i = 1, np1
            ntot = ntot + 1
            do j = 1, isp(1) - 1
               y(ntot,1,j) = y(i,1,j)
            end do 
            do j = 1, isp(2) - 1
               y(ntot,2,j) = y(i,1,j)
            end do 
            do j = 1, isp(3) - 1
               y(ntot,3,j) = xy(j,h)
            end do 
         end do 
      end do 
 
99    end

      subroutine cart (ksite,sname)
c---------------------------------------------------------------------
c      xmax2   |\              cartesian subdivision onto 1-, 2- and 3-
c              |  \            dimensional simplexes. isp(ksite) is number of
c              |    \          vertices, in 1-dimensional case 
c      xinc2   |      \        ordinates of vertices are xmin1 & xmax1,
c              |        \      in 2-dimensional case coordinates
c              |          \    are (xmin1, xmin2), (xmax1, xmin2) & 
c      xmin2   |            \  (xmin1, xmax2).
c              ---------------
c           xmin1    xinc1    xmax1

c      ksite - the site of the solution sname to be represented.
c      iscp  - site dimension

c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'
 
      parameter (ms1=msp-1)
 
      double precision vco1(673), vco2(673)
 
      integer ibary(4), loop(ms1)
 
      character sname*10
 
      save vco1, vco2, ibary
 
      common/ cst86 /xy(ms1,k1),yy(k1,mst,ms1),ntot,npairs
     *      / cst108 /wg(27,11),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(msp,msp,msp),isub(27,9,2),imd(3),
     *      imsol(msp,msp,msp),ijk(mst),insp(mst,msp),jsp(mst),
     *      jsitp(mst),iosp(mst),jst(mst),ist(mst),
     *      isp(mst),isite,ispct(mst,msp),jnsp(mst,msp),jsite,iterm,iord
c                               ibary(j) is the number of stars after
c                               jth barycentric subdivision.
      data ibary/7,25,121,673/
c                               (vco1(i), vco2(i)) are the simplicial
c                               coordinates of the stars generated by
c                               barycentric subdivsion.
      data (vco1(i), i = 1, 3)/ 0., 1., 0./
      data (vco2(i), i = 1, 3)/ 1., 0., 0./

      data (vco1(i), i = 4,121)/.500,.500,.0  ,.333,
c                      4  after subdivision:            1
     *.750,.611,.250,.417,.278,.0  ,.167,.111,.0  ,.167,.111,.250,
     *.167,.278,.750,.667,.417,.611,
c                     22  after subdivision:            2
     *.875,.787,.625,.681,.620,.556,.509,.514,.454,.472,.537,.806,
     *.639,.759,.375,.343,.125,.264,.176,.139,.148,.222,.259,.375,
     *.306,.343,.458,.389,.347,.398,.0  ,.037,.0  ,.056,.037,.056,
     *.093,.139,.204,.250,.222,.204,.083,.056,.139,.093,.0  ,.037,
     *.0  ,.056,.037,.056,.093,.139,.204,.250,.222,.204,.083,.056,
     *.139,.093,.125,.176,.375,.264,.343,.389,.398,.347,.343,.250,
     *.306,.259,.083,.139,.222,.148,.875,.787,.625,.681,.620,.458,
     *.556,.509,.375,.514,.454,.500,.472,.537,.833,.806,.639,.759/
c                    118  after subdivision:            3
      data (vco1(i), i = 122,337)/
     *.938,.887,.813,.831,.804,.769,.739,.734,.693,.699,.735,.894,
     *.796,.864,.688,.665,.563,.623,.582,.560,.559,.588,.596,.646,
     *.616,.637,.715,.685,.650,.684,.489,.484,.461,.463,.480,.512,
     *.545,.583,.560,.559,.528,.505,.532,.522,.415,.414,.387,.394,
     *.420,.463,.512,.562,.532,.526,.465,.435,.484,.461,.457,.519,
     *.568,.602,.614,.588,.596,.542,.574,.540,.403,.435,.505,.448,
     *.864,.796,.753,.653,.713,.688,.625,.699,.670,.708,.685,.725,
     *.903,.880,.782,.855,.438,.406,.313,.359,.323,.296,.285,.303,
     *.295,.310,.336,.421,.366,.410,.188,.184,.063,.150,.100,.088,
     *.105,.157,.198,.271,.227,.239,.257,.213,.220,.230,.077,.116,
     *.133,.157,.179,.185,.216,.208,.213,.188,.069,.074,.144,.096,
     *.225,.255,.281,.296,.299,.282,.281,.250,.269,.253,.194,.213,
     *.241,.216,.354,.350,.396,.359,.378,.380,.369,.345,.323,.292,
     *.310,.309,.319,.338,.324,.327,.479,.452,.437,.428,.424,.382,
     *.407,.387,.312,.373,.341,.333,.338,.355,.444,.449,.394,.429,
     *.0  ,.012,.0  ,.019,.012,.019,.031,.046,.068,.074,.068,.019,
     *.046,.031,.0  ,.012,.0  ,.019,.012,.019,.031,.046,.068,.083,
     *.074,.068,.028,.019,.046,.031,.059,.088,.114,.130,.133,.116/
      data (vco1(i), i = 338,553)/
     *.114,.083,.102,.086,.028,.046,.074,.049,.207,.227,.262,.269,
     *.253,.213,.179,.125,.157,.151,.153,.185,.171,.170,.292,.262,
     *.208,.227,.207,.185,.170,.171,.151,.167,.157,.179,.278,.269,
     *.213,.253,.042,.059,.125,.088,.114,.153,.130,.133,.125,.116,
     *.114,.083,.102,.086,.028,.046,.074,.049,.0  ,.012,.0  ,.019,
     *.012,.019,.031,.046,.068,.074,.068,.019,.046,.031,.0  ,.012,
     *.0  ,.019,.012,.019,.031,.046,.068,.083,.074,.068,.028,.019,
     *.046,.031,.059,.088,.114,.130,.133,.116,.114,.083,.102,.086,
     *.028,.046,.074,.049,.207,.227,.262,.269,.253,.213,.179,.125,
     *.157,.151,.153,.185,.171,.170,.292,.262,.208,.227,.207,.185,
     *.170,.171,.151,.167,.157,.179,.278,.269,.213,.253,.042,.059,
     *.125,.088,.114,.153,.130,.133,.125,.116,.114,.083,.102,.086,
     *.028,.046,.074,.049,.063,.100,.188,.150,.184,.213,.230,.220,
     *.239,.227,.198,.088,.157,.105,.313,.323,.438,.359,.406,.421,
     *.410,.366,.336,.271,.310,.295,.257,.296,.303,.285,.452,.428,
     *.424,.407,.387,.373,.341,.333,.338,.355,.444,.449,.394,.429,
     *.378,.359,.350,.338,.327,.324,.309,.312,.310,.323,.382,.380,
     *.345,.369,.292,.281,.208,.255,.225,.213,.216,.241,.253,.292/
      data (vco1(i), i = 554,673)/
     *.269,.281,.319,.296,.282,.299,.042,.077,.125,.116,.133,.194,
     *.157,.179,.250,.185,.216,.208,.213,.188,.069,.074,.144,.096,
     *.938,.887,.813,.831,.804,.769,.739,.734,.693,.699,.735,.894,
     *.796,.864,.688,.665,.563,.623,.582,.560,.559,.588,.596,.646,
     *.616,.637,.715,.685,.650,.684,.479,.489,.437,.484,.461,.463,
     *.480,.512,.545,.583,.560,.559,.528,.505,.532,.522,.396,.415,
     *.354,.414,.387,.394,.420,.463,.512,.562,.532,.526,.465,.435,
     *.484,.461,.417,.457,.583,.519,.568,.602,.614,.588,.596,.542,
     *.574,.540,.403,.435,.505,.448,.917,.864,.750,.796,.753,.653,
     *.713,.688,.625,.699,.670,.708,.685,.725,.903,.880,.782,.855/
c                    670  after subdivision:            4
      data (vco2(i), i = 4,121)/.500,.0  ,.500,.333,
c                      4  after subdivision:            1
     *.250,.278,.750,.417,.611,.750,.667,.611,.250,.417,.278,.0  ,
     *.167,.111,.0  ,.167,.167,.111,
c                     22  after subdivision:            2
     *.125,.176,.375,.264,.343,.389,.398,.347,.343,.306,.259,.139,
     *.222,.148,.625,.620,.875,.681,.787,.806,.759,.639,.537,.375,
     *.472,.454,.458,.556,.514,.509,.875,.787,.625,.681,.620,.556,
     *.509,.514,.454,.500,.472,.537,.833,.806,.639,.759,.375,.343,
     *.125,.264,.176,.139,.148,.222,.259,.375,.306,.343,.458,.389,
     *.347,.398,.0  ,.037,.0  ,.056,.037,.056,.093,.139,.204,.250,
     *.222,.204,.083,.056,.139,.093,.0  ,.037,.0  ,.056,.037,.083,
     *.056,.093,.250,.139,.204,.250,.222,.204,.083,.056,.139,.093/
c                    118  after subdivision:            3
      data (vco2(i), i = 122,337)/
     *.063,.100,.188,.150,.184,.213,.230,.220,.239,.227,.198,.088,
     *.157,.105,.313,.323,.438,.359,.406,.421,.410,.366,.336,.271,
     *.310,.295,.257,.296,.303,.285,.452,.428,.424,.407,.387,.373,
     *.341,.333,.338,.355,.444,.449,.394,.429,.378,.359,.350,.338,
     *.327,.324,.309,.312,.310,.323,.382,.380,.345,.369,.281,.255,
     *.225,.213,.216,.241,.253,.292,.269,.281,.319,.296,.282,.299,
     *.077,.116,.133,.194,.157,.179,.250,.185,.216,.208,.213,.188,
     *.069,.074,.144,.096,.563,.582,.688,.623,.665,.685,.684,.650,
     *.637,.616,.596,.560,.588,.559,.813,.804,.938,.831,.887,.894,
     *.864,.796,.735,.646,.699,.693,.715,.769,.734,.739,.864,.796,
     *.753,.713,.688,.699,.670,.708,.685,.725,.903,.880,.782,.855,
     *.568,.519,.457,.435,.448,.505,.540,.625,.574,.596,.653,.602,
     *.588,.614,.354,.387,.396,.414,.415,.435,.461,.484,.526,.542,
     *.532,.512,.403,.394,.463,.420,.479,.489,.437,.484,.461,.465,
     *.463,.480,.562,.512,.545,.583,.560,.559,.528,.505,.532,.522,
     *.938,.887,.813,.831,.804,.769,.739,.734,.693,.699,.735,.894,
     *.796,.864,.688,.665,.563,.623,.582,.560,.559,.588,.596,.646,
     *.616,.637,.715,.685,.650,.684,.489,.484,.461,.463,.480,.512/
      data (vco2(i), i = 338,553)/
     *.545,.583,.560,.559,.528,.505,.532,.522,.415,.414,.387,.394,
     *.420,.463,.512,.562,.532,.526,.465,.435,.484,.461,.417,.457,
     *.583,.519,.568,.602,.614,.588,.596,.542,.574,.540,.403,.435,
     *.505,.448,.917,.864,.750,.796,.753,.653,.713,.688,.625,.699,
     *.670,.708,.685,.725,.903,.880,.782,.855,.438,.406,.313,.359,
     *.323,.296,.285,.303,.295,.310,.336,.421,.366,.410,.188,.184,
     *.063,.150,.100,.088,.105,.157,.198,.271,.227,.239,.257,.213,
     *.220,.230,.077,.116,.133,.157,.179,.185,.216,.208,.213,.188,
     *.069,.074,.144,.096,.225,.255,.281,.296,.299,.282,.281,.250,
     *.269,.253,.194,.213,.241,.216,.354,.350,.396,.359,.378,.380,
     *.369,.345,.323,.292,.310,.309,.319,.338,.324,.327,.479,.452,
     *.437,.428,.424,.382,.407,.387,.312,.373,.341,.333,.338,.355,
     *.444,.449,.394,.429,.0  ,.012,.0  ,.019,.012,.019,.031,.046,
     *.068,.074,.068,.019,.046,.031,.0  ,.012,.0  ,.019,.012,.019,
     *.031,.046,.068,.083,.074,.068,.028,.019,.046,.031,.059,.088,
     *.114,.130,.133,.116,.114,.083,.102,.086,.028,.046,.074,.049,
     *.207,.227,.262,.269,.253,.213,.179,.125,.157,.151,.153,.185,
     *.171,.170,.292,.262,.208,.227,.207,.185,.170,.171,.151,.167/
      data (vco2(i), i = 554,673)/
     *.157,.179,.278,.269,.213,.253,.042,.059,.125,.088,.114,.153,
     *.130,.133,.125,.116,.114,.083,.102,.086,.028,.046,.074,.049,
     *.0  ,.012,.0  ,.019,.012,.019,.031,.046,.068,.074,.068,.019,
     *.046,.031,.0  ,.012,.0  ,.019,.012,.019,.031,.046,.068,.083,
     *.074,.068,.028,.019,.046,.031,.042,.059,.125,.088,.114,.130,
     *.133,.116,.114,.083,.102,.086,.028,.046,.074,.049,.208,.207,
     *.292,.227,.262,.269,.253,.213,.179,.125,.157,.151,.153,.185,
     *.171,.170,.292,.262,.208,.227,.207,.185,.170,.171,.151,.167,
     *.157,.179,.278,.269,.213,.253,.042,.059,.125,.088,.114,.153,
     *.130,.133,.125,.116,.114,.083,.102,.086,.028,.046,.074,.049/
c                    670  after subdivision:            4
c---------------------------------------------------------------------
      npairs = 0

	if (imd(ksite).eq.0) then 
c                                  for cartesian subdivision
c                                  check limits and counters:
         do i = 1, isp(ksite) - 1
            call xchk (xmn(ksite,i),xmx(ksite,i),xnc(ksite,i),sname)
            xnot = 0d0
            do j = 1, isp(ksite)
               if (i.ne.j) xnot = xnot + xmn(ksite,j)
            end do 
            if (xmx(ksite,i) .gt. 1d0 - xnot) then 
               call warn (170, x, imod, 'CART')
               xmx(ksite,i) = 1d0 - xnot
            end if 
            loop(i) = 1 
     *              + idint ((xmx(ksite,i)-xmn(ksite,i))/xnc(ksite,i))
         end do 
      end if

      if (isp(ksite).ne.2) goto 30
c                              one-dimensional simplex:
      if (imd(ksite).eq.0) then
c                              cartesian imd = 0:
         x = xmn(ksite,1)
c
         do j = 1, loop(1)
            npairs = npairs+1
c                              generate compositions:
            xy(1,npairs) = x
            x = x + xnc(ksite,1)
         end do 
c
      else if (imd(ksite).eq.1) then
         if (xmx(ksite,1).eq.0d0.or.dabs(xmx(ksite,1)).gt.1d0) then
            delx = 1d0
            x0 = 0d0
         else if (xmx(ksite,1).gt.0d0) then
            delx = xmx(ksite,1)
            x0 = 0d0
         else if (xmx(ksite,1).lt.0d0) then
            delx = dabs(xmx(ksite,1))
            x0 = 1d0 - delx
         end if 
c                              symmetric stretching imod = 1:
         b = xmn(ksite,1)
         if (b.lt.1d0) call error (207,b,j,sname)
         bm1 = b - 1d0
         bp1 = b + 1d0
 
         dx = 1d0 / (xnc(ksite,1) - 1d0)
         x = dx
         loop(1) = idint (xnc(ksite,1))  - 1
c                              first loop generates phases
c                              with xx(1)  =  0 through 0.5 .
         do j = 1, loop(1)
            npairs = npairs + 1
c                              generate compositions:
            t1 = (bp1/bm1)**(1d0-x)
            xy(1,npairs) = x0 + (0.5 * (bp1-bm1*t1)/(1d0 +t1))*delx
            x = x + dx
         end do 
c                              second loop generates phases
c                              with y = 0.5 through 1 - xmin.
         loop(1) = loop(1) - 1
         jpairs = loop(1)
         do j = 1, loop(1)
            npairs = npairs + 1
c                              generate compositions:
            xy(1,npairs) = 2d0*x0 + delx - xy(1,jpairs)
            jpairs  = jpairs - 1
         end do 

      else if (imd(ksite).eq.2) then
c                              assymmetric stretching imod = 2:
         b = xmn(ksite,1)
         if (b.lt.1d0) call error (207,b,j,sname)
         bm1 = b - 1d0
         bp1 = b + 1d0
 
         dx = 1d0 / (xnc(ksite,1) - 1d0)
         x = dx
         loop(1) = idint (xnc(ksite,1)) - 1
c                              with x = xmin through xmax
         do j = 1, loop(1)
            npairs = npairs + 1
c                              generate compositions:
            t1 = (bp1/bm1)**(1d0-x)
            xy(1,npairs) = xmx(ksite,1) * (bp1-bm1*t1)/(1d0+t1)
            x = x + dx
         end do 
c                              value of imod is illegal, 
c                              write error msg:
      else if (imd(ksite).gt.3)  then
 
         call error (169,b,imod,'CART')
 
      end if
 
      goto 99

30    if (isp(ksite).ne.3) goto 20 
c                              two dimensional simplex:
      if (imd(ksite).eq.0) then
c                              cartesian subdivision:
         x = xmn(ksite,1)

         do i = 1, loop(1)

            y = xmn(ksite,2)
            if (x + y .gt. 1d0) goto 99 
 
            do j = 1, loop(2)
c                               generate compositions:
               if (x + y .gt. 1d0) goto 10
               npairs = npairs + 1
               if (npairs.gt.k1) call error (180,b,k1,'CART')
               xy(1,npairs) = x
               xy(2,npairs) = y
               y = y + xnc(ksite,2)
            end do 
 
10          x = x + xnc(ksite,1)

         end do  

      else if (imd(ksite).gt.0.and.imd(ksite).lt.5) then
c                               barycentric subdivision of ternary
c                               solutions, imod = # of iterations.
         npairs = ibary(imd(ksite))
         if (npairs.gt.k1) call error (180,b,k1,'CART')
         dx1 = xmx(ksite,1) - xmn(ksite,1) 
         dx2 = xmx(ksite,2) - xmn(ksite,2)
         do j = 1, npairs
            xy(1,j) = xmn(ksite,1) + ( vco1(j) - xmn(ksite,1) ) * dx1
            xy(2,j) = xmn(ksite,2) + ( vco2(j) - xmn(ksite,2) ) * dx2
         end do 
 
      else if (imod.gt.4) then
c                              illegal value of imod, write error msg:
           call error (169,b,imd(ksite),'CART')
 
      end if

      goto 99 

20    if (isp(ksite).eq.4) then

         if (imd(ksite).eq.0) then 
c                              cartesian subdivision:
            x = xmn(ksite,1)

            do i = 1, loop(1)

               y = xmn(ksite,2)
               if (x + y .gt. 1d0) goto 99
 
               do j = 1, loop(2)

                  z = xmn(ksite,3)
                  if (x + y + z .gt. 1d0) goto 50 

                  do  k = 1, loop(3)
c                               generate compositions:
                      npairs = npairs + 1
                      if (npairs.gt.k1) call error (180,b,k1,'CART')
                      xy(1,npairs) = x
                      xy(2,npairs) = y
                      xy(3,npairs) = z
                      z = z + xnc(ksite,3)
                      if (x + y + z .gt. 1d0) goto 60 

                  end do 
                 
60                y = y + xnc(ksite,2)
                  if (x + y .gt. 1d0) goto 50

               end do 

50             x = x + xnc(ksite,1)

            end do 

         else 

            call error (169,b,imod,'CART')

         end if

      else

           call error (169,b,imod,'CART')

      end if 

99    end

      subroutine xchk (xmin, xmax, xinc, tname) 

      implicit double precision (a-g,o-z),integer (h-n)

      character tname*10

      if (xmax.gt.1d0) then 
         call warn (21,xmax,1,tname)
         xmax = 1d0
      end if 
      if (xmin.lt.0d0) then 
         call warn (22,xmin,1,tname) 
         xmin = 0d0
      end if 
      if (xmax.lt.xmin) then
         call warn (23,xmax,1,tname)
         xmax = 1d0
         xmin = 0d0
      end if 
      if (xinc.le.0d0) then 
         call warn (23,xinc,1,tname)
         xinc = 1d0       
      end if
      end 

      subroutine zchk (z, tname)

      implicit double precision (a-g,o-z),integer (h-n)

      character tname*10

      if (z.gt.0d0.and.z.le.1d-3) then
         z = 0d0
      else if (z.ge.0.999.and.z.lt.1d0) then
         z = 1d0
      else if (z.ge.0d0.and.z.le.1d0) then 
         return
      else if (z.gt.1d0.and.z.lt.1.01) then
         z = 1d0
      else if (z.gt.-.01) then
         z = 0d0
      else 
         call error (125,z,1,tname)
      end if
      end 

      subroutine assptx
c---------------------------------------------------------------------
c assptx assigns the current value of v1 and v2 to the array ptx and
c increments the ordinate counter ipt2.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst32 /ptx(l5),ipt2
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5
     *      / cst5 /v(l2),tr,pr,r,ps/ cst63 /delv(l2)
  
      ipt2 = ipt2 + 2

      if (ipt2.gt.l5) ipt2 = l5

      ptx(ipt2-1) = v(iv1)
      ptx(ipt2)   = v(iv2)
 
      end
 
      subroutine concrt
c---------------------------------------------------------------------
c concrt determines convergence criteria and limits for reasonable
c solutions for the routine univeq. the array delt is also used in
c the routine slope.
 
c input:  dv - array of default search increments.
c         vmax,vmin - arrays containing the maximum and minimum values
c           of the independent and dependent intensive variables.
c         iv - array indexes variables in vmax, vmin, and output arrays
c output: ulim, blim - arrays with upper and lower limits for reasonable
c           solutions (vmax,vmin+/-4*dv)
c         delt - array containing the finite difference increments
c           (vmax-vmin/10**5), delt also serves as the test value
c           for convergence.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst62 /blim(l2),ulim(l2),dgr
     *      / cst9 /vmax(l2),vmin(l2),dv(l2)
c---------------------------------------------------------------------
      do i = 1, 5
         if (dv(i).lt.0d0) call error (34,dv(i),i,'CONCRT')
         if (i.eq.3) then 
            ulim(i) = vmax(i)
            blim(i) = vmin(i)
         else if (i.eq.1.or.i.eq.2) then
            ulim(i) = vmax(i)+dv(i)
            blim(i) = vmin(i)-dv(i)
            if (blim(i).lt.0d0) blim(i) = 1d0 
         else 
            ulim(i) = vmax(i)+dv(i)
            blim(i) = vmin(i)-dv(i)
         end if 
         ddv = vmax(i)-vmin(i)
         if (ddv.lt.0d0) call error (35,ddv,i,'CONCRT')
      end do 
 
      end
 
      subroutine conver (g,s,v,a,b,c,d,e,f,gg,b1,b2,b3,b4,b5,b6,b7,b8,
     *                   b9,b10,b11,b12,tr,pr)
c---------------------------------------------------------------------
c convert thermodynamic equation of state from a pr tr reference state
c to get polynomial coefficients
c---------------------------------------------------------------------
 
      implicit double precision (a-g,o-z),integer (h-n)

c                              b8 is K' in the murgnahan EoS
c                              or -beta in exponential volumetric
c                              functions.
      if (b8.eq.0d0) then
c                              normal polynomial vdp term:
 
c                              first compute G*
         g  = g
c                              add the integral of sdt at tr,pr:
     *       + s * tr - a * tr - b * tr * tr / 2d0 + c / tr
     *       - d * tr**3 / 3.d0 - 2d0 * e * dsqrt(tr)
     *       - f * dlog(tr) + gg / tr / tr / 2d0 + f
c                              add the constant components of the
c                              integral -vdp at (t,pr):
     *       - v * pr + b2 * tr * pr + b4 * pr * pr / 2d0 
     *       - b6 * pr**3/3.d0 + b7 * tr * tr * pr
c                              S* (T)
         s  = a - b2*pr - s + a*dlog(tr) + b*tr - c/tr/tr/2d0
     *        + d * tr * tr / 2d0 - 2d0 * e / dsqrt(tr) - f/tr
     *        - gg / tr**3 / 3.d0
c                              V* (P)
         v  = v - b2 * tr - b4 * pr  - b7 * tr * tr
c                              a*  (-T log T)
c        a  = a
c                              b*  (-T*T)
         b  = b7 * pr + b / 2d0
c                              c*  (-1/T)
         c  = c / 2d0
c                              d*  (-T**3)
         d  = d / 6.d0
c                              e*  (sqrt T)
         e  = 4.d0 * e
c                              f*  (log T)
c        f  = f
c                              gg* (-1/T**2)
         gg = gg/6.d0
c                              b2* (PT)
c        b2 = b2
c                              b3* ((P-Pr) exp (T / c1))
c        b3 = b3
c                              b4* (P**2)
         b4 = b4/2d0
c                              b5* (exp (P/ c2))
c        b5 = c2 * b5
c                              b6* (P**3)
         b6 = b6/3.d0
c                              b7* (P*T**2)
c        b7 = b7
      else
c                              nonlinear EoS's:

c                              thermal terms:
         g  = g
c                              add the integral of sdt at tr,pr:
     *       + s * tr - a * tr - b * tr * tr / 2d0 + c / tr
     *       - d * tr**3 / 3d0 - 2d0 * e * dsqrt(tr)
     *       - f * dlog(tr) + gg / tr / tr / 2d0 + f
c                              S* (T)
         s  = a - s + a * dlog(tr) + b * tr - c / tr / tr /2d0
     *        + d * tr * tr / 2d0 - 2d0 * e / dsqrt(tr) - f / tr
     *        - gg / tr**3 / 3d0
c                              these terms are as above
         b  = b / 2d0
         c  = c / 2d0
         d  = d / 6.d0
         e  = 4d0 * e
         gg = gg/6.d0
c                              volume terms:
         if (b8.gt.0d0) then
c                              mughnahan vdp
            b1 = v*b1 
            b2 = v*b2/2d0 
            b3 = v*b3 
            b4 = -v*b4 
            b5 = 2d0*v*b5 
            v  = v - (b1-b2*tr)*tr - b3*dlog(tr) - b4/tr - b5*dsqrt(tr) 
            b6 = b6 
            b7 = b7 
c           b8 = kappap 
c                                 these are unnecessary, but may cut time?
            b9 = (1d0-1d0/b8) 
            b10 = b8*pr 
            b11 = 1d0/b8 
            b12 = b8 - 1d0

         else 
c                              gottschalk, b3 = alpha, -b8 = beta: 
            b1 = -v / b8 / dexp (b3*tr)
      
         end if 

      end if 
 
      end

      subroutine disord (tk,d0,d1,d2,d3,d4,d5,d6,tr,tmax,g)
c----------------------------------------------------------------------
c compute t-dependent disorder contribution, g is the
c gibbs energy of disordering, tr is the t of onset of
c disordering, tmax is the t of complete disorder.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      common/ cst5 /p,v(4),pr,z(3)
 
      g = 0d0
      if (tk .lt. tr) return
      tt  = tk
      if (tk.gt.tmax) tt = tmax
 
      dh = d0 * (tt - tr)
     *     + 2d0 * d1 * (tt**0.5 - tr**0.5)
     *     - d2 * (1d0 / tt - 1d0 / tr)
     *     + d4 * dlog(tt/tr)
     *     + d5 * (tt*tt - tr*tr) / 2d0
     *     + d6 * (tt**3 - tr**3) / 3d0
 
      ds = d0 * dlog(tt/tr)
     *     - 2d0 * d1 * (tt**(-0.5) - tr**(-0.5))
     *     - d2 * (1d0/tt/tt - 1d0/tr/tr) / 2d0
     *     - d4 * (1d0/tt - 1d0 / tr)
     *     + d5 * (tt - tr)
     *     + d6 * (tt*tt - tr*tr) / 2d0
 
      gss = dh - (tk * ds)
      if (d3.ne.0d0) gss = gss + dh/d3 * (p - pr)
      end
 
      subroutine getphi (name,ibase,icmpn,*)
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      character*8 name, oname, record*1
 
      common/ cst41  /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst43  /therm(k4),comp(k0),atwt(k0),idh2o,idco2,ikind
     *      / cst202 /tm(m7,m6),td(m8),ilam,idiso,lamin,idsin
     *      / cst207 /ctrans(k0,k5),ictr(k5),itrans/ cst318 /emodu(10)
 
      save oname
 
      data oname/' '/
 
30    read (n2,1010,end=90,err=98) record
      if (record.eq.' ') then
c                          check for comments, i.e., data 
c                          with a blank 1st character 
         goto 30
      else  
         backspace (n2)
         read (n2,1020,end=90,err=98)
     *                     name, ibase, ikind, ilam, idiso
      end if 

      read (n2,*,err=98) (comp(i), i=1, icmpn), 
     *                   (therm(i), i= 1, k14)
c                               do component transformation if
c                               itrans is not zero
      if (itrans.gt.0) then
 
         do i = 1, itrans
            it = ictr(i)
            if (comp(it).ne.0d0) then
c                                ct is how much of the new
c                                component is in the phase.
               ct =  comp(it) / ctrans(it,i)
 
               do j = 1, icmpn
                  comp(j) = comp(j) - ct * ctrans(j,i)
               end do 
 
               comp(it) = ct
            end if 
         end do
      end if
 
      if (ilam.ne.0) then
c                               determine number of transitions from
c                               flag ilam:
         jlam = ilam

         if (ilam.gt.3) jlam = ilam - 3
         if (ilam.gt.6) jlam = ilam - 6
         if (ilam.gt.9) jlam = ilam - 9
 
         do i= 1, jlam
            read (n2,*,err=98) (tm(j,i), j = 1, m7 - 2)
         end do 
      end if
 
      if (idiso.ne.0) read (n2,*,err=98) td

      if (ikind.ne.0) read (n2,*,err=98) emodu
  
      oname = name
 
      return
 
90    return 1

98    if (oname.ne.' ') then
         call error (23,ct,i,oname)
       else
         call error (23,ct,i,'  none  ')
       end if

1010  format (a1)
1020  format (a8,i2,i2,i2,i2)
 
      end
 
      subroutine chkphi (ichk,name,tot,icout,ibase,icmpn,*)
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      character*8 name, exname
 
      integer icout(k0)
 
      common/ cst43 /therm(k4),comp(k0),atwt(k0),idh2o,idco2,ikind
     *      / cst6 /icomp,istct,iphct,icp/ cst36 /exname(h8)
     *      / cst37 /ixct,iexyn/ cst42 /ic(k5),idbase
 
c                               reject the data if wrong data base:
      if (ibase.ne.idbase) goto 90
c                               reject the data if excluded in input:
      do i = 1, ixct
         if (name.eq.exname(i)) goto 90
      end do
c                               reject phases which have a component
c                               not included in the input:
      do i= 1, icmpn
         if (icout(i).eq.0.and.comp(i).ne.0d0) goto 90
      end do
c                               reject phases with negative compositions
      do j = 1, icmpn
         if (comp(j).lt.0d0) then
            if (ichk.eq.0) call warn (13,r,j,name)
            goto 90
         end if
      end do 
c                               the following is not executed by build:
c                               if ichk ne 0 reject phases that consist entirely
c                               of saturated components: 
      if (ichk.eq.0) return
c                               reject phases with null composition, in case
c                               a user puts one in by accident
      tot = 0d0
      do j = 1, icp
         tot = tot + comp(ic(j))
      end do 

      if (tot.eq.0d0) goto 90

      goto 99

90    return 1
 
99    end
 
      subroutine lamlan (dg,rkt,ld,ltype)
c---------------------------------------------------------------------
c     calculate the extra energy of a lamdba transition using  the
c     Landau model (Holland and Powell).
 
c     input variables
 
c                        t = temperature in k
c                        p = pressure in bar
c                        ld = pointer to phase in therlm
c                        rkt = bulk modulus
 
c     returned - dg - difference in free energy due to the transition
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst203 /therdi(m8,m9),therlm(m7,m6,k9)
     *      / cst5 /p,t,xco2,u1,u2,tr,pr,r,ps
     *      / cst1 /thermo(k4,k10),uf(2),us(h5)

      lct = ltype - 9
c                              don't do this stuff until TJBH
c                              has a consistent policy:

c                              create a constant which contains
c                              the 0 to Tr contribution to the
c                              free energy:
c               q = (1.-(tr/tc))**0.25
c               if (tr.gt.tc) q = 0.
c                              maximum enthalpy of disorder:
c               therlm (3,j,lamin) = 2./3.*smax*tc
c                              enthalpy of disorder at tr:
c               therlm (4,j,lamin) = 2.*smax*tc * 
c     *                              (q**6/6. - q*q/2. + 1./3.)
c                              entropy of disorder at tr:
c               therlm (5,j,lamin) = 

 
      dg = 0d0

      do i = 1, lct
 
         tc = therlm(1,i,ld) + therlm(3,i,ld)*(p-pr)
 
         smax = therlm (2,i,ld)

         if (t.gt.tc) then
c                                  fully disordered:
            dg = dg + therlm(4,i,ld) - t*therlm(5,i,ld)
 
         else
c                                  partially disordered:
            q2 = dsqrt(1d0-t/tc)
c                                  bulk modulus:
            dg = dg   
c                                  hr-t*sr
     *           + therlm(4,i,ld) - t*therlm(5,i,ld)   
c                                  G(landau)
     *           + smax * (q2*(t-tc) + tc*q2**3/3d0)
 
         end if
c                                  if pressure dependent
c                                  add the vdp term:
c                                  pressure dependent 
c                                  landau transitions are
c                                  only in the mughnahan 
c                                  versions of HP data.
         if (therlm(3,i,ld).ne.0d0) then
            dg = dg 
c                                  vt*kt/3
     *           + (therlm(6,i,ld) + therlm(7,i,ld)*t
     *              + therlm(8,i,ld)*dsqrt(t))*rkt/3d0
c                                  *[(1+4p/kt)^(3/4) - 1]
c                                  this approximation is only
c                                  good for pr near 0.
     *             *((1d0 + 4d0*p/rkt)**(0.75d0) - 1d0)
         end if 

 
      end do
 
      end

      function gtrans (id,j)
c-----------------------------------------------------------------------
c gtrans computes the reference pressure free energy of a compound 
c aboves its jth transition. 
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst203 /therdi(m8,m9),therlm(m7,m6,k9)
     *      / cst5 /p,t,xco2,u1,u2,tr,pr,r,ps/ cst44 /dlogt,dsqrtt
 
      gtrans = therlm(12,j,id)
c                                    -sdt
     *       + t * (therlm(3,j,id) - therlm(5,j,id) * dlogt
     *       - t * (therlm(6,j,id) + therlm(8,j,id) * t))
     *       - (therlm(7,j,id) + therlm(11,j,id) / t) / t
     *       + therlm(9,j,id) * dsqrtt + therlm(10,j,id)*dlogt

      end 
 
      subroutine lamhel (p,t,g,vdp,ld,ltype)
c---------------------------------------------------------------------
c     calculate the extra energy of a lamdba transition using model
c     of helgeson et al 1978 (AJS).
 
c     there is something seriously wrong in this routine!!!!
 
c     input variables
c                        p = pressure in bars
c                        t = temperature in k
c                        ld = pointer to phase in therlm
c                        g = initial free energy
 
c     returned - g - modified phase free energy
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst203 /therdi(m8,m9),therlm(m7,m6,k9)/ cst5 /v(6),pr,u(2)
 
      lct = ltype - 3
c                                 T<T lowest transition, ignore
c                                 possibility of < clapeyron slope
c                                 and exit:
      if (t.lt.therlm(1,1,ld)) goto 99

      do 10 i = 1, lct 
         if (t.lt.therlm(1,i,ld)) then 

            if (i.eq.1) then
c                                 T<T lowest transition, ignore
c                                 possibility of < clapeyron slope
c                                 and exit:
                goto 99
             else
c                                 at the i-1 th transition:
                goto 20
             end if 
          else 
             jtran = i 
          end if 
10    continue

20    g = gtrans(ld,jtran) + vdp 
c                                 add vtrans*dp terms, this is
c                                 only set up for one transition
c                                 with a non-zero clapeyron.
c                                 should write warning. 
      if (therlm(2,1,ld).eq.0d0) goto 99

      trtp = therlm(1,1,ld) + (p-pr)/therlm(2,1,ld)
      dt = t - therlm(1,1,ld)

      if (t .gt. trtp) then
c                                     1 bar polymorph is stable at p,
c                                     p*thermo(3,id) is 0 bar standard
c                                     state pdv polymorph contribution &
c                                     therlm(4,j,ld) is Delta V of trans
c                                     contribution.
         pstar = dt*therlm(2,1,ld) + pr
 
         g = g + (p-pstar)*therlm(4,1,ld)
 
      else
c                                     the 1 bar polymorph isn't stable.
         g = g + dt * therlm(2,1,ld) * therlm(4,1,ld)
 
      end if
 
99    end
 
      subroutine lamqtz (p,t,g,du,ld,id)
c---------------------------------------------------------------------
c     calculate the extra energy of a lamdba transition using model
c     of helgeson et al 1978 (AJS). eq. (114) (corrected for the
c     misplaced parenthesis and the sign on c sub alpha), this is
C     probably bogus for coesite note that only one transition
c     is allowed.
 
c     input variables
c                        p = pressure in bars
c                        t = temperature in k
c                        ld = pointer to phase in therlm
c                        g = initial free energy
 
c     constants
c                Tr = 298.15 K
c                Pr = 1. bar
c                S(lope) = 38.5 bar/K
c                ba = 0.066 j/bar
c                aa = 549.82 K
c                ca = -4.973d-6 j/bar/bar
c                trt = 848. K
 
c     returned - g - modified phase free energy
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst1   /thermo(k4,k10),uf(2),us(h5)
     *      / cst203 /therdi(m8,m9),therlm(m7,m6,k9)
 
      save trt, tr, pr, s, aa, ba, ca , vb
 
      data trt, tr, pr, s, aa, ba, ca, vb / 848., 298.15, 1d0,
     *                  38.5, 549.82, 0.066, -4.973d-6, 2.372 /
 
c      trtp = trt + (p-pr) / s
 
      if (t.gt.trt) then
         ps = (t-trt) * therlm(2,1,ld) + pr
      else
         ps = pr
      end if
c                                      if above the ref P transition T
c                                      do the cp integration:
      if (t.gt.trt) g = therlm(8,1,ld) + (p-ps) * thermo(3,id)
     *                - therlm(3,1,ld) * (t-trt) + du
     *     + therlm(5,1,ld) * (t - trt - t * dlog(t/trt))
     *     - (therlm(7,1,ld) + therlm(6,1,ld) * t * trt * trt)
     *     * (t - trt)**2 / 2d0 / t / trt / trt
 
c                                      now add in pdv terms to
c                                      the free energy, note that
c                                      pdv term for the ref polymorph
c                                      is already in:
 
      pdv  =  vb * (ps-pr)
     *      - ca * ( (2. * pr * (p-ps) - (p * p - ps * ps) ) / 2.
     *              + s * (t-tr) * (p-ps) )
     *      + s * (ba + aa*ca*s) * (t-tr)
     *                           * dlog ((aa+p/s) / (aa + ps/s))
 
      g = g + pdv
 
      end
 
      subroutine lamubc (p,t,gspk,k,ltype)
c---------------------------------------------------------------------
c     calculate the extra energy of a lamdba transition using model
c     of berman and brown (1985, contribs. miner. petro.)
 
c     input variables
c                        p = pressure in bars
c                        t = temperature in k
c                        k = pointer to phase in therlm
 
c     returned - gspk - energy because of lambda spike
 
c     modified from perkins et al 1986 (computers and geosciences).
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst203 /therdi(m8,m9),therlm(m7,m6,k9)
c---------------------------------------------------------------------
      gspk=0d0
 
      do 10 j = 1, ltype
 
         tq1bar = therlm(3,j,k)
 
         if (tq1bar.eq.0d0) goto 10
 
            tr    = therlm(7,j,k)
            teq   = therlm(4,j,k) * (p-1d0) + tq1bar
            ctrans = tq1bar - teq
            tr9    = tr - ctrans
 
            if (t.lt.tr9) goto 10
 
            aspk2 = therlm(1,j,k)
            bspk2 = therlm(2,j,k)
            dvdtr = therlm(5,j,k)
            dvdp  = therlm(6,j,k)
            dstr  = therlm(8,j,k) / tq1bar
            abspk = therlm(9,j,k)
 
            if (t .gt. teq) then
               t9 = teq
            else
               t9 = t
            end if
 
            ct2 = ctrans * ctrans
            ct3 = ct2 * ctrans
 
            a1 = aspk2 * ctrans + 2. * abspk * ct2 + bspk2 * ct3
            b1 = aspk2 + 4. * abspk * ctrans + 3. * bspk2 * ct2
            c1 = 2. * abspk + 3. * ctrans * bspk2
 
            t92 = t9 * t9
            t93 = t9 *t92
            tr92 = tr9 * tr9
            tr93 = tr92 * tr9
 
            dhspk = a1 * (t9 - tr9)
     *            + b1 * (t92 - tr92) / 2.
     *            + c1 * (t93 - tr93) / 3.
     *            + bspk2 * (t9*t93 - tr93*tr9) / 4.
 
            dsspk = a1 * (dlog(t9) - dlog(tr9))
     *            + b1 * (t9 - tr9)
     *            + c1 * (t92 - tr92) / 2.
     *            + bspk2 * (t93 - tr93) / 3.
 
            gspk = gspk - (t9 * dsspk) + dhspk
 
            if (t.gt.teq) gspk = gspk - (dstr + dsspk ) * (t-teq)
 
            gspk = gspk + dvdtr * (p-1d0) * (t9-tr)
     *                  + dvdp * ((p*p-1d0)/2d0 - (p-1d0))
 
10    continue
      end
 
      subroutine loadit (id)
c---------------------------------------------------------------------
c loadit loads descriptive data for phases and species (name,comp,
c and therm) into the appropriate arrays (names,comps,thermo,vf,
c and vs).  the arguement 'id' indexes the phase in the arrays.
c note that loadit also computes reference state constants which
c are dependent on the state function being used and on its
c analytical expression.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      character*8 names, name, cmpnt*5, dname*40
      double precision z(13)
 
      common/ cst60  /ipoint,imyn/ cst319 /emod(10,k10),iemod(k1),kmod
     *      / cst61  /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst1   /thermo(k4,k10),uf(2),us(h5)
     *      / cst10  /iff(2),idss(h5),ifug,ifyn,isyn
     *      / cst8   /names(k1)/ cst318 /emodu(10)
     *      / cst5   /p,t,xco2,u1,u2,tr,pr,r,ps/ cst12 /cp(k5,k1)
     *      / cst6   /icomp,istct,iphct,icp/ csta6 /name
     *      / cst40  /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / csta5  /dname(8),cmpnt(k0)/ cst42 /ic(k5),idbase
     *      / cst202 /tm(m7,m6),td(m8),ilam,idiso,lamin,idsin
     *      / cst203 /therdi(m8,m9),therlm(m7,m6,k9)
     *      / cst43  /therm(k4),comp(k0),atwt(k0),idh2o,idco2,ikind
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / cst204 /ltyp(k1),lmda(k1),idis(k1)
     *      / cst304 /sxs(k13),exces(3,k1),jend(h9,k12),ixp(k1),ifp(k1)
c---------------------------------------------------------------------
      if (iphct.gt.k10) call error (180,r,k10,'LOADIT')
      ipoint = iphct
      
c                               load name and phase flag
      names(id) = name
      iemod(id) = ikind
      ikp(id) = 0
      atw(id) = 0d0
c                               set fluid flag:
      if (therm(3).eq.0d0.and.ifyn.eq.1) then
         ifp(id) = 1       
      else
         ifp(id) = 0
      end if
c                               load stoichiometry of components.
      do i = 1, icomp
         atw(id) = comp(ic(i)) * atwt(i) + atw(id)
         cp(i,id) = comp(ic(i))
      end do 
c                               load elastic props if present
      if (iemod(id).ne.0) then 
c                               kmod initialized 0 in main.
         kmod = 1        
         do i = 1, 10
            emod(i,id) = emodu(i)
         end do
      end if 
c                               compute reference state constants,
c                               etc..
      do i = 1, k4
         thermo(i,id) = therm(i)
      end do 
 
      call conver (thermo(1,id),thermo(2,id),thermo(3,id),thermo(4,id),
     *             thermo(5,id),thermo(6,id),thermo(7,id),thermo(8,id),
     *             thermo(9,id),thermo(10,id),thermo(11,id),
     *             thermo(12,id),thermo(13,id),thermo(14,id),
     *             thermo(15,id),thermo(16,id),thermo(17,id),
     *             thermo(18,id),thermo(19,id),thermo(20,id),
     *             thermo(21,id),thermo(22,id),tr,pr)
 
      if (tr.eq.0d0) then
         thermo(1,id) = therm(1)
         thermo(2,id) = therm(2)
      end if
c                              lmda transitions:
      idis(id) = 0
      lmda(id) = 0
      if (ilam.ne.0) then
         lamin=lamin+1
 
         if (ilam.ge.10.and.ilam.le.12) then
c                              holland and powell, landau model:
            lct = ilam - 9
            do 140 j = 1, lct       

               smax = tm(2,j)
               t0 = tm(1,j)
               vmax = tm(3,j)
               a0 = therm(11)

               therlm(1,j,lamin) = t0
               therlm(2,j,lamin) = smax
c                              this makes therlm(3) dt/dp
               therlm(3,j,lamin) = tm(3,j)/tm(2,j)
c                              qr2,qr4
               
               qr2 = dsqrt (1d0 - tr/t0)
               qr4 = (1d0 - tr/t0)**(1.5)/3.d0
c                              href:
               therlm(4,j,lamin) = smax*t0*(qr2-qr4)
c                              sref:
               therlm(5,j,lamin) = smax*qr2
c                              vT coefficients:  
c                              constant:
               therlm(6,j,lamin) = vmax*qr2*
     *                             (1d0-a0*tr+20d0*a0*dsqrt(tr))
c                              *T coefficient:
               therlm(7,j,lamin) = vmax*qr2*a0
c                              *sqrt(T) coeff:
               therlm(8,j,lamin) = -20d0*vmax*qr2*a0
c                              TJBH said that he is not yet
c                              consistent in correcting the 
c                              reference state props for the
c                              landau effect, accordingly I
c                              assume here that the effect is
c                              not incorporated. 

c                              this stuff is no longer possible with
c                              hollands variable tc formulation:

c                              create a constant which contains
c                              the 0 to Tr contribution to the
c                              free energy:
c               q = (1.-(tr/tc))**0.25
c               if (tr.gt.tc) q = 0.
c                              maximum enthalpy of disorder:
c               therlm (3,j,lamin) = 2./3.*smax*tc
c                              enthalpy of disorder at tr:
c               therlm (4,j,lamin) = 2.*smax*tc * 
c     *                              (q**6/6. - q*q/2. + 1./3.)
c                              entropy of disorder at tr:
c               therlm (5,j,lamin) = smax * (1. - q*q)
140         continue

 
 
         else if (ilam.le.3) then
c                              ubc:
            do 90 j = 1, ilam
 
               therlm(1,j,lamin)=tm(1,j)*tm(1,j)
               therlm(2,j,lamin)=tm(2,j)*tm(2,j)
               therlm(9,j,lamin)=tm(1,j)*tm(2,j)
 
               do 80 k = 3, 8
80                therlm(k,j,lamin)=tm(k,j)
90          continue
 
         else if (ilam.gt.3.and.ilam.lt.8) then
c                              helgeson:
            p = pr
c                              now convert paramters:
            do 100 k = 1, ilam - 3
c                              load into therlm:
               therlm(1,k,lamin) = tm(1,k)
               therlm(2,k,lamin) = tm(2,k)

               do 120 j = 4, 10
120               therlm(j+1,k,lamin) = tm(j,k)

               t = tm(1,k)
               if (k.eq.1) then
c                              set transition type to null
c                              for call to gphase
                  lmda(id) = 0
               else 
                  lmda(id) = lamin
                  ltyp(id) = 2 + k
               end if
c                              g at trt:
               call updumy
               call gphase (id,gph)
               therlm(12,k,lamin) = gph
c                              delta v trans:
               therlm(4,k,lamin) = 0d0
               if (tm(2,k).ne.0.) therlm(4,k,lamin) = tm(3,k)/tm(2,k)
c                              s + st at trt:
               t = t + 1d-3
               call updumy
               call gphase (id,gph)
               therlm(3,k,lamin) = tm(3,k) -
     *                             (gph - therlm(12,k,lamin))/1d-3
c                              streamline the eos:
               do j = 1, 13
                  z(j) = 0d0 
               end do 

100            call conver (therlm(12,k,lamin),therlm(3,k,lamin),
     *          z(1),therlm(5,k,lamin),therlm(6,k,lamin),
     *          therlm(7,k,lamin),therlm(8,k,lamin),therlm(9,k,lamin),
     *          therlm(10,k,lamin),therlm(11,k,lamin),
     *          z(2),z(3),z(4),z(5),z(6),z(7),z(8),
     *          z(9),z(10),z(11),z(12),z(13),tm(1,k),pr)

         end if
 
         lmda(id)=lamin
         ltyp(id)=ilam
 
      end if
c                              t dependent order: load berman and brown
c                              parameters, (this should be cleaned up)
      if (idiso.ne.0) then
         idsin = idsin + 1
         idis(id) = idsin
         do 70 j = 1, m8
70          therdi(j,idsin) = td(j)
      end if
 
      end

      subroutine difgx (x,fact,index,iord,isub)
c---------------------------------------------------------------------
c subroutine to evaluate the function P(X)+(1-xi)*diff(P(X),xi)
c where P(X) is the product x(isub(1))*x(isub(2))*...,
c isub contains the indexes of x in P(X), and index is the
c value of i, the value is returned as difgx. iord is the maximum
c order possible for P(X).
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
  
      double precision x(4)

      integer isub(4), jnot(3), iord
 
      if (x(index).eq.1d0.or.x(index).eq.0d0) 
     *              call error (27,x(1),iord,'IMBECILLE!')
c                               find the order of P(X) = jord,
c                               and count the number of occurences
c                               of x(index) = ixi.
      ixi = 0 
      ini = 0 
      do 10 i = 1, iord
         if (isub(i).ne.0) then 
c                               assume 0 subscripts at end of isub
            jord = i
            if (isub(i).eq.index) then 
               ixi = ixi + 1
            else 
               ini = ini + 1
               jnot(ini) = isub(i)
            end if 
         end if 
10    continue
c                               2nd order parameter:
      if (jord.eq.2) then

         xm = 1. - x(index)
         
         if (ixi.eq.0) then 

            fact = -x(isub(1))*x(isub(2))

         else if (ixi.eq.1) then 

            fact = x(jnot(1))*x(index) 
     *           + (xm*x(jnot(1))-2.*x(index)*x(jnot(1))) / xm

         else 

            call error (999,fact,jord,'difgx')

         end if 
c                               3rd order parameter:
      else if (jord.eq.3) then

         if (ixi.eq.0) then 

            fact = -2.*x(isub(1))*x(isub(2))*x(isub(3))

         else if (ixi.eq.1) then 

            fact = -2.*x(isub(1))*x(isub(2))*x(isub(3)) 
     *           + x(jnot(1))*x(jnot(2))

         else if (ixi.eq.2) then 
       
            fact = 2.*(x(index)*x(jnot(1)) - x(index)**2*x(jnot(1)))

         else 

            call error (999,fact,jord,'difgx')

         end if 

      else if (jord.eq.4) then 
c                               4th order parameter:
         if (ixi.eq.0) then 

            call error (999,fact,jord,'difgx')

         else if (ixi.eq.1) then 

             fact = -2.*x(index)**2*x(jnot(1))*x(jnot(2))*x(jnot(3))
     *            + 2. * x(isub(1))*x(isub(2))*x(isub(3))*x(isub(4))

         else if (ixi.eq.2) then 

             fact = 2.*x(index)*x(jnot(1))*x(jnot(2))
     *            - 3.*x(index)**2*x(jnot(1))*x(jnot(2))

         else if (ixi.eq.3) then 

             fact = 3.*(x(jnot(1))*x(index)**2 
     *            - x(jnot(1))*x(index)**3)
 
         end if 
      end if 
      end

      subroutine univeq (i,ier)
c---------------------------------------------------------------------
c univeq computes the equilibrium condition of a univariant reaction.
c using a finite difference newton-raphson approximation. the method
c will fail when the derivative of the state function, with respect to
c the state variable v(i), goes to infinity.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst62 /blim(l2),ulim(l2),dgr/ cst5  /v(l2),tr,pr,r,ps
     *      / cst87 /delt(l2),dtol,utol,ptol
 
      ier = 0
c     jer = 0 
 
      vi = v(i)
      del = delt(i)
      b = blim(i)
      u = ulim(i)
c                                  phase composition special case:
      if (i.eq.3) then 
         if (vi.lt.1d1*del) then 
            del = dabs(vi)/1d1
         else if (1d0-vi.lt.1d1*del) then 
            del = dabs(1d0-vi)/1d1
         end if 
      end if 
 
      if (vi+dabs(del).gt.u.or.vi-dabs(del).lt.b) goto 30
 
5     do j = 1, 100

         call grxn (gr)
         v(i) = vi + del
         call incdep (i,v(i))
         call grxn (dgr) 
         dgr = dgr - gr
         if (dgr.eq.0d0) goto 20
 
         delv = gr*del/dgr

c         if (dabs(dgr).lt.1d-6*utol) then
c                                this used to work when 
c                                dgr was compared to utol, but
c                                perhaps it's now superfluous.
c                                a better test might be to compare
c                                gr/dgr to 1?

c                                actually, this should probably be
c                                a test for divergence, i.e., gr/dgr
c                                > something.
c            if (dabs(delv).lt.del) goto 10
c            jer = jer + 1
c            del = 1d1 * del
c            if (jer.gt.5) then 
c               call warn (999,dgr,jer,'UNIVEQ')

         vi = vi - delv

         if (vi+dabs(del).gt.u.or.vi-dabs(del).lt.b) goto 30

         v(i) = vi
         call incdep (i,v(i))
c                                 test for convergence
c         if (dabs(gr).lt.utol) goto 99
c                                 used to be on the basis of utol,
c                                 but this allows uniform relative error.
         if (dabs(delv).lt.del) goto 99
c               goto 20
c            end if
c            v(i) = vi
c            goto 5
c         end if 
      end do 
c                                  error code 1: iv and dv must be
c                                  switched
20    ier = 1
      return
c                                  error code 2: value too far out of
c                                  range, refine iv increment.
30    ier = 2
99    end

      subroutine unver (g,s,v,a,b,c,d,e,f,gg,
     *                  b1,b2,b3,b4,b5,b6,b7,b8,tr,pr)
c---------------------------------------------------------------------
c convert thermodynamic equation of state from a 0-0 reference state
c to a pr-tr reference state.
c---------------------------------------------------------------------
 
      implicit double precision (a-g,o-z),integer (h-n)

      gg = 6d0 * gg
      e  = e / 4d0
      d  = 6d0 * d
      c  = 2d0 * c
c                                b8 is K'
      if (b8.eq.0d0) then 
c                                normal vdp term:
         b6 = 3d0 * b6
         b4 = 2d0 * b4
         b  = 2d0 * (b - b7*pr)
         v  = v + b2 * tr + b4 * pr + b7 * tr * tr
         s  = -1d0 * ( s - ( a - b2 * pr  + a * dlog(tr) + b * tr
     *        - c / tr / tr /2d0
     *        + d * tr * tr / 2d0 - 2d0 * e / dsqrt(tr) - f / tr
     *        - gg / tr**3 / 3.d0 ) )
         g  = g - ( s * tr - a * tr - b * tr * tr / 2d0 + c / tr
     *        - d * tr**3 / 3.d0 - 2d0 * e * dsqrt(tr)
     *        - f * dlog(tr) + gg / tr / tr / 2d0 + f
     *        - v * pr + b2 * tr * pr + b4 * pr * pr / 2d0 
     *        - b6 * pr**3/3.d0 + b7 * tr * tr * pr )
      else 

         b  = 2d0 * b

         s  = -1d0 * ( s - ( a + a * dlog(tr) + b * tr
     *        - c / tr / tr /2d0
     *        + d * tr * tr / 2d0 - 2d0 * e / dsqrt(tr) - f / tr
     *        - gg / tr**3 / 3d0 ) )
         g  = g - ( s * tr - a * tr - b * tr * tr / 2d0 + c / tr
     *        - d * tr**3 / 3d0 - 2d0 * e * dsqrt(tr)
     *        - f * dlog(tr) + gg / tr / tr / 2d0 + f)

         if (b8.gt.0d0) then 
c                                 murghnahan vdp:
            v  = v + (b1+b2*tr)*tr + b3*dlog(tr) + b4/tr + b5*dsqrt(tr)
            b1 = b1 / v
            b2 = 2d0 * b2/v
            b3 = b3 / v
            b4 = -b4 / v
            b5 = b5 / v / 2d0  
         else 
c                                 v = f(exponential(beta))
c                                 zero b1, so users don't get confused.
            b1 = 0d0
         end if  

      end if 
 
      end

 
      subroutine updumy
c---------------------------------------------------------------------
c dummy call to replace uproj in loadit
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      common/ cst5  /p,t,xco2,u1,u2,tr,pr,r,ps/ cst44 /dlogt,dsqrtt
 
c                           compute isobaric isothermal constants.
      if (t.lt.0d0.or.tr.eq.0d0) then
        dlogt=0d0
        dsqrtt=0d0
      else
        dlogt=dlog(t)
        dsqrtt=dsqrt(t)
      end if
 
      end
 
c-----------------------------------------------------------------------
 
c PLIB -  subroutines common to FRENDLY and VERTEX.
 
c-----------------------------------------------------------------------
 
      subroutine slope (iv1,iv2,s)
c---------------------------------------------------------------------
c slope computes the slope (div(1)/div(2)) of a univariant
c equilibria by approximating the derivatives dg/div(1) and
c dg/div(2) by finite differences.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      integer iv(2)

      double precision dg(2)
 
      common/ cst5 /v(l2),tr,pr,r,ps/ cst62 /blim(l2),ulim(l2),dgr
     *      / cst87 /delt(l2),dtol,utol,ptol
 
      iv(1) = iv1
      iv(2) = iv2
      call grxn (gr)

      do i = 1, 2
         v(iv(i)) = v(iv(i)) + delt(iv(i))
         call incdep (iv(i),v(iv(i)))
         call grxn (gval)
         dg(i) = (gval-gr)/delt(iv(i))
c                                    note the possibility exists here
c                                    for the value of an intensive
c                                    parameter to excede allowed
c                                    limits.  although this is very
c                                    unlikely a limit test like the
c                                    one done in 'univeq' could be
c                                    used.
         v(iv(i)) = v(iv(i)) - delt(iv(i))
         call incdep (iv(i),v(iv(i)))
      end do 

      s = -dg(2) / dg(1)
 
      end
 
      subroutine switch (div,ivi,ivd,jer)
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst5 /v(l2),tr,pr,r,ps/ cst9 /vmax(l2),vmin(l2),dv(l2)
     *      / cst24 /ipot,jv(l2),iv(l2)/ cst31 /vn(k2,k5+1),irct,ird
c---------------------------------------------------------------------
c                                  reset intensive variables
      call reptx 
      vo = v(ivi)
c                                  determine the sign for the
c                                  ivd increment
      call slope(ivd,ivi,s)
      jer = 0
c
      if (s.eq.0d0) then
c                                  a zero slope shouldn't occur
c                                  someplace is a bug
         jer = 1
         return
      end if
c
      odiv = div
      div = dv(ivd)
      if (odiv.gt.0d0) goto 10
      if (s.gt.0d0) div = -dv(ivd)
      goto 20
10    if (s.lt.0d0) div = -dv(ivd)
c                                 estimate a new value for v(ivi)
20    v(ivi) = v(ivi)+div/s
      call incdep (ivi,v(ivi)) 
c                                 switch variables
      if ((v(ivi).gt.vmin(ivi)).and.(v(ivi).lt.vmax(ivi))) goto 30
      div = div/5.d0
      v(ivi) = vo
      call incdep (ivi,v(ivi)) 
      goto 20

30    iovd = ivd
      ivd = ivi
      ivi = iovd

      end

      subroutine unblnk (text)
c------------------------------------------------------------------- 
c unblnk - subroutine to remove blanks from text
 
c     text - character string 
c     jchar - length of unblanked character string, 0 on input 
c             if unknown.
c-------------------------------------------------------------------
 
      character text*(*), bitsy(400)*1 

      nchar = len(text)
 
      read (text,1000) (bitsy(i), i = 1, nchar)
c                                 scan for blanks:
      ict = 0

      do 10 i = 1, nchar
         if (bitsy(i).eq.' ') goto 10
         ict = ict + 1
         bitsy(ict) = bitsy(i)
10    continue

      write (text,1000) (bitsy(i), i = 1, ict)
 
1000  format (400a1)
      end

      subroutine enblnk (text)
c------------------------------------------------------------------- 
c enblnk - scan text to first blank and cut remaining text.
 
c     text - character string 
c     jchar - length of unblanked character string, 0 on input 
c             if unknown.
c-------------------------------------------------------------------
 
      character text*(*), bitsy(400)*1 
 
      read (text,1000) bitsy
c                                 scan for blanks:
      ict = 0
      nchar = len(text)

      do i = 1, nchar
         if (bitsy(i).eq.' ') goto 10
         ict = ict + 1
      end do 

10    write (text,1000) (bitsy(i), i = 1, ict),(' ',i = ict+1, nchar)
 
1000  format (400a1)
      end

      subroutine deblnk (text)
c------------------------------------------------------------------- 
c deblnk - scan text and delete multiple blank characters
 
c     text - character string 
c-------------------------------------------------------------------
 
      character text*(*), bitsy(400)*1 
 
      read (text,1000) bitsy
c                                 scan for blanks:
      ict = 0
      nchar = len(text)

      do i = 2, nchar
         if (bitsy(i).eq.' '.and.bitsy(i-1).eq.' ') goto 10 
         ict = ict + 1
         bitsy(ict) = bitsy(i)
10    end do 

      write (text,1000) (bitsy(i), i = 1, ict),(' ',i = ict+1, nchar)
 
1000  format (400a1)
      end

      subroutine mertxt (text,text1,text2)
c------------------------------------------------------------------- 
c mertxt - merge text strings cutting trailing blanks, if text1 eq 
c          blank then pad by an initial 40 blanks.
 
c     text - character string 
c-------------------------------------------------------------------
 
      character text*(*), text1*(*), text2*(*), bitsy(400)*1 

      if (text1.eq.' ') then
         write (text,1010) text2
      else 
         nchar1 = len(text1) 
         read (text1,1000) (bitsy(i), i = 1, nchar1)  
         do i = 2, nchar1
            if (bitsy(i).eq.' '.and.bitsy(i-1).eq.' ') then 
               nchar1 = i - 1
               goto 10 
            end if 
         end do 
c                                  put 5 blanks between 1st and 
c                                  2nd strings
10       do i = nchar1, nchar1 + 5
            bitsy(i) = ' '
         end do 

         nchar1 = nchar1 + 5 
         nchar2 = nchar1 + len(text2) 
         nchar3 = len(text)

         if (nchar2.gt.nchar3) then 
            nchar2 = nchar3 - nchar1
         end if 

         if (nchar2.gt.400) then 
            nchar2 = 400 - nchar1
         end if 

         read (text2,1000) (bitsy(i), i = nchar1, nchar2)  
         write (text,1000) (bitsy(i), i = 1, nchar2),
     *                     (' ',i = nchar2+1, nchar3)
      end if 

1010  format (40x,a)
1000  format (400a1)

      end



      subroutine inblnk (text,char)
c------------------------------------------------------------------- 
c inblnk - scan text to last '/' or '\' and insert char after.
 
c     text - character string 
c-------------------------------------------------------------------
 
      character text*(*), bitsy(400)*1, char*1 
 
      read (text,1000) bitsy
c                                 scan for blanks:
      ict = 0
      nchar = len(text)

      do i = nchar,1,-1
c                                 this line may cause problems
c                                 on some operating systems that 
c                                 recognize the backslash as an escape
c                                 character.
         if (bitsy(i).eq.'/') goto 10
         bitsy(i+1) = bitsy(i)
      end do 

      i = 0

10    bitsy(i+1) = char

      write (text,1000) (bitsy(i), i = 1, nchar)
 
1000  format (400a1)
      end

      subroutine reblnk (text)
c------------------------------------------------------------------- 
c reblnk - subroutine to replace blanks followed by a character
c          with an underscore.
 
c     text - character string 
c     jchar - length of unblanked character string, 0 on input 
c             if unknown.
c-------------------------------------------------------------------
 
      character*8 text, bitsy(8)*1 
 
      read (text,1000) bitsy
c                                 scan for blanks:
      ict = 0

      do 10 i = 1, 7

         if (i.eq.1.and.bitsy(i).eq.' ') goto 10 

         if (bitsy(i).eq.' '.and.bitsy(i+1).ne.' ') then
            ict = ict + 1
            bitsy(ict) = '_'
         else if (bitsy(i).eq.' ') then 
            goto 10
         else
            ict = ict + 1
            bitsy(ict) = bitsy(i)
         end if

10    continue 
 
      bitsy(ict+1) = bitsy(8)

      write (text,1000) (bitsy(i), i = 1, ict + 1)
 
1000  format (8a1)
      end


      subroutine reptx
c-----------------------------------------------------------------------
c reset - resets indendent potentials to the last known stable condition 
c along a univariant curve 
c----------------------------------------------------------------------- 

      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'
    
      common/ cst5  /v(l2),tr,pr,r,ps/ cst32 /ptx(l5),ipt2
     *      / cst24 /ipot,jv(l2),iv(l2)

      v(iv(1)) = ptx(ipt2-1)
      call incdep (iv(1),v(iv(1)))
      v(iv(2)) = ptx(ipt2)
      call incdep (iv(2),v(iv(2)))
   
      end

      subroutine toop (id,gex)
c-----------------------------------------------------------------------
c giulio, here is a dummy routine to serve as a template, be careful
c of the type conventions that i use (i.e., all reals are implicit
c double) and also not to change any of the variables in the common
c blocks. you have to keep the three common blocks cts6, cst5 and cst12
c for input, the variables that will concern you are 

c on input:
c           r - the gas constant, J/mol-K
c           t - temperature (K)
c           p - pressure (bar)
c           tr - reference t, as in the thermodynamic data file header.
c           pt - reference p, dido.
c           icp - the number of components.
c           id - a pointer to the phase composition
c           cp(i,id) - the mol fraction of the ith component in composition id.
c on output:
c           gex - the excess free energy in J (i.e., in excess of a mechanical
c                 mixture of the endmembers) per mol of components.

c N.B. the components appear in the same order that they are entered in 
c the input file and this must also be identical to the order in the 
c solution model data file. i think build will always make SIO2 the 
c first component, overriding your choice. so to be safe anywhere you
c enter components expect SIO2 to be the 1st composition, then enter 
c the remaining components in order of the cation atomic weight, e.g., 
c cp(1,id) will always be X(SIO2) in the id'th composition, cp(2,id) will
c be X(CA0) if there are no other components.

c----------------------------------------------------------------------- 

      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst5 /p,t,xco2,u1,u2,tr,pr,r,ps/ cst12 /cp(k5,k1)
     *      / cst6   /icomp,istct,iphct,icp

      gex = 0d0

      do i = 1, icp
         if (cp(i,id).ne.0d0) gex = gex + r*t*cp(i,id)*log(cp(i,id))
      end do 

      end 
         
      subroutine unlam (tm,id,lct)
c---------------------------------------------------------------------
      implicit double precision (a-g,o-y),integer (h-m)
 
      include 'perplex_parameters.h'
  
      double precision tm(m7,m6),z(9)

      common/ cst203 /therdi(m8,m9),therlm(m7,m6,k9)
     *      / cst100 /dlnfo2,elag,gz,gy,gx,ibuf,hu,hv,hw,hx
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst204 /ltyp(k1),lmda(k1),idis(k1)
     *      / cst5 /p,t,xco2,u1,u2,tr,pr,r,ps
c-----------------------------------------------------------------------
                                 
      ilam = ltyp(id)
      jd = lmda(id)
      lct = 0 

      do i = 1, 10
         do j = 1, 3
            tm(i,j) = 0d0
         end do 
      end do 

      if (ilam.ne.0.and.(ilam.lt.4.or.ilam.gt.7)) then
   
         ilam = ltyp(id)
         jd = lmda(id)
         if (ilam.le.3) then
            lct = ilam
            do j = 1, ilam
               tm(1,j) = dsqrt (therlm(1,j,jd))
               tm(2,j) = dsqrt (therlm(2,j,jd)) 
            end do         
         else if (ilam.gt.9.and.ilam.lt.13) then
            lct = ilam - 9
            do j = 1, ilam - 9
               tm(1,j) = therlm(1,j,jd)
               tm(2,j) = therlm(2,j,jd)
               tm(3,j) = therlm(3,j,jd) *therlm(2,j,jd) 
            end do 
         end if
      end if

      if (ilam.gt.3.and.ilam.lt.7) then

         lct = ilam - 3
         p = pr
         do i = lct, 1 , -1
c                              get s transition:
c                              load into therlm:
            tm(1,i) = therlm(1,i,jd) 
            tm(2,i) = therlm(2,i,jd) 

            do j = 5, 11
               tm(j-1,i) = therlm(j,i,jd)  
            end do 
            t = tm(1,i)

            if (i.eq.1) then
c                              set transition type to null
c                              for call to gphase 
               ltyp(id) = 0
            else 
               ltyp(id) = 2 + i
            end if
c    c                         -s at trt:
            call updumy
            call gphase (id,g1)
            t = t + 1d-3
            call updumy
            call gphase (id,gph)
            tm(3,i) =  (gph - g1)/1d-3

            g0 = therlm(12,i,jd)
            s0 = therlm(3,i,jd)
            do k = 1, 9
               z(k) = 0d0
            end do 

            call unver (g0,s0,z(1),tm(4,i),tm(5,i),tm(6,i),tm(7,i),
     *                  tm(8,i),tm(9,i),tm(10,i),z(2),z(3),z(4),z(5),
     *                  z(6),z(7),z(8),z(9),tm(1,i),pr) 

            tm(3,i) = s0 + tm(3,i)
         end do 

         ltyp(id) = ilam
         lmda(id) = jd

      end if
      
      end 

      subroutine outlam (tm,jd,lct)
c---------------------------------------------------------------------
      implicit double precision (a-g,o-y),integer (h-m)
 
      include 'perplex_parameters.h'
  
      double precision tm(m7,m6)

      common/ cst203 /therdi(m8,m9),therlm(m7,m6,k9)
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
c-----------------------------------------------------------------------
      if (lct.ne.0) write (n2,1000) ((tm(i,j),i=1,10),j= 1,lct)

      if (jd.ne.0) write (n2,1000) (therdi(j,jd), j = 1, m8)

1000  format (5(g13.7,1x))

      end 
 
      subroutine incdep (ind,var) 
c--------------------------------------------------------------------
c incdep computes the dependent variable if defined (idep).

c  ind  - is the index of the current variable. 
c  iind - is the index of the independent variable that v(idep) is a 
c         function of.
c  var  - is the value of the current variable.
c--------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst5  /v(l2),tr,pr,r,ps
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5
     *      / cst316 /c0,c1,c2,c3,c4,iind,idep

      if (ind.eq.iind) then 

         v(idep) = c0 + var*(c1 + var*(c2 + var*(c3 + c4*var)))
 
      end if

      end 

      double precision function depvar (var) 
c--------------------------------------------------------------------
c depvar computes the dependent variable from the independent variable
c var
c--------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      common/ cst316 /c0,c1,c2,c3,c4,iind,idep

      depvar = c0 + var*(c1 + var*(c2 + var*(c3 + c4*var)))

      end 


